@@ -1,17 +1,17 @@
-#ifndef _LC_APPLICATION_H_
-#define _LC_APPLICATION_H_
+#pragma once
 
 #include "lc_array.h"
 
 class Project;
 class lcPiecesLibrary;
 
-enum lcLightingMode
+enum lcShadingMode
 {
-	LC_LIGHTING_UNLIT,
-	LC_LIGHTING_FAKE,
-	LC_LIGHTING_FULL,
-	LC_NUM_LIGHTING_MODES
+	LC_SHADING_WIREFRAME,
+	LC_SHADING_FLAT,
+	LC_SHADING_DEFAULT_LIGHTS,
+	LC_SHADING_FULL,
+	LC_NUM_SHADING_MODES
 };
 
 class lcPreferences
@@ -21,32 +21,33 @@ public:
 	void SaveDefaults();
 
 	int mMouseSensitivity;
-	lcLightingMode mLightingMode;
+	lcShadingMode mShadingMode;
 	bool mDrawAxes;
 	bool mDrawEdgeLines;
 	float mLineWidth;
 	bool mDrawGridStuds;
-	lcuint32 mGridStudColor;
+	quint32 mGridStudColor;
 	bool mDrawGridLines;
 	int mGridLineSpacing;
-	lcuint32 mGridLineColor;
+	quint32 mGridLineColor;
 	bool mFixedAxes;
 };
 
-class lcApplication
+class lcApplication : public QApplication
 {
-	Q_DECLARE_TR_FUNCTIONS(lcApplication);
+	Q_OBJECT
 
 public:
-	lcApplication();
+	lcApplication(int& Argc, char** Argv);
 	~lcApplication();
 
 	void SetProject(Project* Project);
-	bool Initialize(int argc, char *argv[], const char* LibraryInstallPath, const char* LDrawPath, bool& ShowWindow);
+	bool Initialize(QList<QPair<QString, bool>>& LibraryPaths, bool& ShowWindow);
 	void Shutdown();
 	void ShowPreferencesDialog();
+	void SaveTabLayout() const;
 
-	bool LoadPiecesLibrary(const char* LibPath, const char* LibraryInstallPath, const char* LDrawPath);
+	bool LoadPartsLibrary(const QList<QPair<QString, bool>>& LibraryPaths, bool OnlyUsePaths, bool ShowProgress);
 
 	void SetClipboard(const QByteArray& Clipboard);
 	void ExportClipboard(const QByteArray& Clipboard);
@@ -57,25 +58,23 @@ public:
 	QByteArray mClipboard;
 
 protected:
-	void ParseIntegerArgument(int* CurArg, int argc, char* argv[], int* Value);
-	void ParseStringArgument(int* CurArg, int argc, char* argv[], char** Value);
+	QString GetTabLayoutKey() const;
 };
 
-extern lcApplication* g_App;
+extern lcApplication* gApplication;
 
 inline lcPiecesLibrary* lcGetPiecesLibrary()
 {
-	return g_App->mLibrary;
+	return gApplication->mLibrary;
 }
 
 inline Project* lcGetActiveProject()
 {
-	return g_App->mProject;
+	return gApplication->mProject;
 }
 
 inline lcPreferences& lcGetPreferences()
 {
-	return g_App->mPreferences;
+	return gApplication->mPreferences;
 }
 
-#endif // _LC_APPLICATION_H_
