@@ -32,7 +32,8 @@ lcContext::lcContext()
 	mTexCoordEnabled = false;
 	mColorEnabled = false;
 
-	mTexture = 0;
+	mTexture2D = 0;
+	mTexture2DMS = 0;
 	mLineWidth = 1.0f;
 #ifndef LC_OPENGLES
 	mMatrixMode = GL_MODELVIEW;
@@ -40,8 +41,6 @@ lcContext::lcContext()
 #endif
 
 	mFramebufferObject = 0;
-	mFramebufferTexture = 0;
-	mDepthRenderbufferObject = 0;
 
 	mColor = lcVector4(0.0f, 0.0f, 0.0f, 0.0f);
 	mWorldMatrix = lcMatrix44Identity();
@@ -69,22 +68,24 @@ void lcContext::CreateShaderPrograms()
 #define LC_VERTEX_OUTPUT "varying "
 #define LC_PIXEL_INPUT "varying "
 #define LC_PIXEL_OUTPUT
+#define LC_SHADER_PRECISION
 #else
 #define LC_SHADER_VERSION "#version 300 es\n#define texture2D texture\n"
 #define LC_VERTEX_INPUT "in "
 #define LC_VERTEX_OUTPUT "out "
 #define LC_PIXEL_INPUT "in mediump "
 #define LC_PIXEL_OUTPUT "#define gl_FragColor FragColor\nout mediump vec4 gl_FragColor;\n"
+#define LC_SHADER_PRECISION "mediump "
 #endif
 #define LC_PIXEL_FAKE_LIGHTING \
-	"	vec3 Normal = normalize(PixelNormal);\n" \
-	"	vec3 LightDirection = normalize(PixelPosition - LightPosition);" \
-	"	vec3 VertexToEye = normalize(EyePosition - PixelPosition);\n" \
-	"	vec3 LightReflect = normalize(reflect(-LightDirection, Normal));\n" \
-	"	float Specular = abs(dot(VertexToEye, LightReflect));\n" \
+	LC_SHADER_PRECISION "	vec3 Normal = normalize(PixelNormal);\n" \
+	LC_SHADER_PRECISION "	vec3 LightDirection = normalize(PixelPosition - LightPosition);" \
+	LC_SHADER_PRECISION "	vec3 VertexToEye = normalize(EyePosition - PixelPosition);\n" \
+	LC_SHADER_PRECISION "	vec3 LightReflect = normalize(reflect(-LightDirection, Normal));\n" \
+	LC_SHADER_PRECISION "	float Specular = abs(dot(VertexToEye, LightReflect));\n" \
 	"	Specular = min(pow(Specular, 8.0), 1.0) * 0.25;\n" \
-	"	vec3 SpecularColor = vec3(Specular, Specular, Specular);\n" \
-	"	float Diffuse = min(abs(dot(Normal, LightDirection)) * 0.6 + 0.65, 1.0);\n"
+	LC_SHADER_PRECISION "	vec3 SpecularColor = vec3(Specular, Specular, Specular);\n" \
+	LC_SHADER_PRECISION "	float Diffuse = min(abs(dot(Normal, LightDirection)) * 0.6 + 0.65, 1.0);\n"
 
 	const char* VertexShaders[LC_NUM_MATERIALS] =
 	{
@@ -180,7 +181,7 @@ void lcContext::CreateShaderPrograms()
 		"uniform sampler2D Texture;\n"
 		"void main()\n"
 		"{\n"
-		"	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
+		LC_SHADER_PRECISION "	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
 		"	gl_FragColor = vec4(MaterialColor.rgb, TexelColor.a * MaterialColor.a);\n"
 		"}\n",
 		// LC_MATERIAL_UNLIT_TEXTURE_DECAL
@@ -191,8 +192,8 @@ void lcContext::CreateShaderPrograms()
 		"uniform sampler2D Texture;\n"
 		"void main()\n"
 		"{\n"
-		"	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
-		"	gl_FragColor = vec4(mix(MaterialColor.xyz, TexelColor.xyz, TexelColor.a), MaterialColor.a);\n"
+		LC_SHADER_PRECISION "	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
+		"	gl_FragColor = mix(MaterialColor, TexelColor, TexelColor.a);\n"
 		"}\n",
 		// LC_MATERIAL_UNLIT_VERTEX_COLOR
 		LC_SHADER_VERSION
@@ -213,7 +214,7 @@ void lcContext::CreateShaderPrograms()
 		"void main()\n"
 		"{\n"
 		LC_PIXEL_FAKE_LIGHTING
-		"	vec3 DiffuseColor = MaterialColor.rgb * Diffuse;\n"
+		LC_SHADER_PRECISION "	vec3 DiffuseColor = MaterialColor.rgb * Diffuse;\n"
 		"	gl_FragColor = vec4(DiffuseColor + SpecularColor, MaterialColor.a);\n"
 		"}\n",
 		// LC_MATERIAL_FAKELIT_TEXTURE_DECAL
@@ -229,9 +230,9 @@ void lcContext::CreateShaderPrograms()
 		"void main()\n"
 		"{\n"
 		LC_PIXEL_FAKE_LIGHTING
-		"	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
-		"	vec3 DiffuseColor = mix(MaterialColor.xyz, TexelColor.xyz, TexelColor.a) * Diffuse;\n"
-		"	gl_FragColor = vec4(DiffuseColor + SpecularColor, MaterialColor.a);\n"
+		LC_SHADER_PRECISION "	vec4 TexelColor = texture2D(Texture, PixelTexCoord);"
+		LC_SHADER_PRECISION "	vec4 DiffuseColor = mix(MaterialColor, TexelColor, TexelColor.a);\n"
+		"	gl_FragColor = vec4(vec3(DiffuseColor) * Diffuse + SpecularColor, DiffuseColor.a);\n"
 		"}\n"
 	};
 
@@ -365,6 +366,7 @@ void lcContext::SetDefaultState()
 	}
 	else
 	{
+#ifndef LC_OPENGLES
 		glEnableClientState(GL_VERTEX_ARRAY);
 		glDisableClientState(GL_NORMAL_ARRAY);
 		glDisableClientState(GL_TEXTURE_COORD_ARRAY);
@@ -374,6 +376,7 @@ void lcContext::SetDefaultState()
 		glNormalPointer(GL_BYTE, 0, nullptr);
 		glTexCoordPointer(2, GL_FLOAT, 0, nullptr);
 		glColorPointer(4, GL_FLOAT, 0, nullptr);
+#endif
 	}
 
 	mNormalEnabled = false;
@@ -387,7 +390,14 @@ void lcContext::SetDefaultState()
 	mVertexBufferOffset = (char*)~0;
 
 	glBindTexture(GL_TEXTURE_2D, 0);
-	mTexture = 0;
+	mTexture2D = 0;
+#ifndef LC_OPENGLES
+	if (gSupportsTexImage2DMultisample)
+	{
+		glBindTexture(GL_TEXTURE_2D_MULTISAMPLE, 0);
+		mTexture2DMS = 0;
+	}
+#endif
 
 	glLineWidth(1.0f);
 	mLineWidth = 1.0f;
@@ -414,7 +424,8 @@ void lcContext::ClearResources()
 {
 	ClearVertexBuffer();
 	ClearIndexBuffer();
-	BindTexture(0);
+	BindTexture2D(0);
+	BindTexture2DMS(0);
 }
 
 void lcContext::SetMaterial(lcMaterialType MaterialType)
@@ -496,13 +507,27 @@ void lcContext::SetSmoothShading(bool Smooth)
 #endif
 }
 
-void lcContext::BindTexture(GLuint Texture)
+void lcContext::BindTexture2D(GLuint Texture)
 {
-	if (mTexture == Texture)
+	if (mTexture2D == Texture)
 		return;
 
 	glBindTexture(GL_TEXTURE_2D, Texture);
-	mTexture = Texture;
+	mTexture2D = Texture;
+}
+
+void lcContext::BindTexture2DMS(GLuint Texture)
+{
+	if (mTexture2DMS == Texture)
+		return;
+
+#ifndef LC_OPENGLES
+	if (gSupportsTexImage2DMultisample)
+	{
+		glBindTexture(GL_TEXTURE_2D_MULTISAMPLE, Texture);
+		mTexture2DMS = Texture;
+	}
+#endif
 }
 
 void lcContext::SetColor(float Red, float Green, float Blue, float Alpha)
@@ -530,108 +555,190 @@ void lcContext::SetInterfaceColor(lcInterfaceColor InterfaceColor)
 	SetColor(gInterfaceColors[InterfaceColor]);
 }
 
-bool lcContext::BeginRenderToTexture(int Width, int Height)
+void lcContext::ClearFramebuffer()
 {
+	if (!mFramebufferObject)
+		return;
+
+	if (gSupportsFramebufferObjectARB)
+		glBindFramebuffer(GL_FRAMEBUFFER, 0);
+#ifndef LC_OPENGLES
+	else if (gSupportsFramebufferObjectEXT)
+		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
+#endif
+
+	mFramebufferObject = 0;
+}
+
+lcFramebuffer lcContext::CreateFramebuffer(int Width, int Height, bool Depth, bool Multisample)
+{
+	lcFramebuffer Framebuffer(Width, Height);
+
 	if (gSupportsFramebufferObjectARB)
 	{
-		glGenFramebuffers(1, &mFramebufferObject);
-		glGenTextures(1, &mFramebufferTexture);
-		glGenRenderbuffers(1, &mDepthRenderbufferObject);
+		int Samples = (Multisample && gSupportsTexImage2DMultisample) ? QGLFormat::defaultFormat().samples() : 1;
 
-		glBindFramebuffer(GL_DRAW_FRAMEBUFFER, mFramebufferObject);
+		glGenFramebuffers(1, &Framebuffer.mObject);
+		glBindFramebuffer(GL_FRAMEBUFFER, Framebuffer.mObject);
 
-		BindTexture(mFramebufferTexture);
-		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, Width, Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
-		glFramebufferTexture2D(GL_DRAW_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, mFramebufferTexture, 0);
+		glGenTextures(1, &Framebuffer.mColorTexture);
+		if (Depth)
+			glGenRenderbuffers(1, &Framebuffer.mDepthRenderbuffer);
 
-		glBindRenderbuffer(GL_RENDERBUFFER, mDepthRenderbufferObject);
-		glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT24, Width, Height);
-		glFramebufferRenderbuffer(GL_DRAW_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, mDepthRenderbufferObject);
-
-		BindTexture(0);
-		glBindFramebuffer(GL_FRAMEBUFFER, mFramebufferObject);
-
-		if (glCheckFramebufferStatus(GL_DRAW_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
+		if (Samples == 1)
 		{
-			EndRenderToTexture();
-			return false;
+			BindTexture2D(Framebuffer.mColorTexture);
+			glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, Width, Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
+			BindTexture2D(0);
+			glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, Framebuffer.mColorTexture, 0);
+
+			if (Depth)
+			{
+				glBindRenderbuffer(GL_RENDERBUFFER, Framebuffer.mDepthRenderbuffer);
+				glRenderbufferStorage(GL_RENDERBUFFER, GL_DEPTH_COMPONENT24, Width, Height);
+				glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, Framebuffer.mDepthRenderbuffer);
+			}
 		}
-
-		return true;
-	}
-
 #ifndef LC_OPENGLES
-	if (gSupportsFramebufferObjectEXT)
-	{
-		glGenFramebuffersEXT(1, &mFramebufferObject); 
-		glGenTextures(1, &mFramebufferTexture); 
+		else
+		{
+			BindTexture2DMS(Framebuffer.mColorTexture);
+			glTexImage2DMultisample(GL_TEXTURE_2D_MULTISAMPLE, Samples, GL_RGBA, Width, Height, GL_TRUE);
+			BindTexture2DMS(0);
+			glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D_MULTISAMPLE, Framebuffer.mColorTexture, 0);
 
-		BindTexture(mFramebufferTexture); 
+			if (Depth)
+			{
+				glBindRenderbuffer(GL_RENDERBUFFER, Framebuffer.mDepthRenderbuffer);
+				glRenderbufferStorageMultisample(GL_RENDERBUFFER, Samples, GL_DEPTH_COMPONENT24, Width, Height);
+				glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, Framebuffer.mDepthRenderbuffer);
+			}
+		}
+#endif
+		
+		if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE)
+			DestroyFramebuffer(Framebuffer);
+
+		glBindFramebuffer(GL_FRAMEBUFFER, mFramebufferObject);
+	}
+#ifndef LC_OPENGLES
+	else if (gSupportsFramebufferObjectEXT)
+	{
+		glGenFramebuffersEXT(1, &Framebuffer.mObject);
+		glGenTextures(1, &Framebuffer.mColorTexture);
+
+		BindTexture2D(Framebuffer.mColorTexture);
 		glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
 		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA8, Width, Height, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
 
-		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFramebufferObject); 
-		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, mFramebufferTexture, 0); 
+		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, Framebuffer.mObject);
+		glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_2D, Framebuffer.mColorTexture, 0);
 
-		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFramebufferObject);
+		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, Framebuffer.mObject);
 
-		glGenRenderbuffersEXT(1, &mDepthRenderbufferObject); 
-		glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, mDepthRenderbufferObject); 
+		glGenRenderbuffersEXT(1, &Framebuffer.mDepthRenderbuffer);
+		glBindRenderbufferEXT(GL_RENDERBUFFER_EXT, Framebuffer.mDepthRenderbuffer);
 		glRenderbufferStorageEXT(GL_RENDERBUFFER_EXT, GL_DEPTH_COMPONENT24, Width, Height);
 
-		glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, mDepthRenderbufferObject); 
+		glFramebufferRenderbufferEXT(GL_FRAMEBUFFER_EXT, GL_DEPTH_ATTACHMENT_EXT, GL_RENDERBUFFER_EXT, Framebuffer.mDepthRenderbuffer);
 
-		BindTexture(0);
-		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFramebufferObject);
+		BindTexture2D(0);
+		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, Framebuffer.mObject);
 
 		if (glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT) != GL_FRAMEBUFFER_COMPLETE_EXT)
-		{
-			EndRenderToTexture();
-			return false;
-		}
+			DestroyFramebuffer(Framebuffer);
 
-		return true;
+		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, mFramebufferObject);
 	}
 #endif
-	
-	return false;
+
+	return Framebuffer;
 }
 
-void lcContext::EndRenderToTexture()
+void lcContext::DestroyFramebuffer(lcFramebuffer& Framebuffer)
 {
 	if (gSupportsFramebufferObjectARB)
 	{
-		glDeleteFramebuffers(1, &mFramebufferObject);
-		mFramebufferObject = 0;
-		glDeleteTextures(1, &mFramebufferTexture);
-		mFramebufferTexture = 0;
-		glDeleteRenderbuffers(1, &mDepthRenderbufferObject);
-		mDepthRenderbufferObject = 0;
-
-		return;
+		glDeleteFramebuffers(1, &Framebuffer.mObject);
+		glDeleteTextures(1, &Framebuffer.mColorTexture);
+		glDeleteRenderbuffers(1, &Framebuffer.mDepthRenderbuffer);
 	}
-
 #ifndef LC_OPENGLES
-	if (gSupportsFramebufferObjectEXT)
+	else if (gSupportsFramebufferObjectEXT)
 	{
-		glDeleteFramebuffersEXT(1, &mFramebufferObject);
-		mFramebufferObject = 0;
-		glDeleteTextures(1, &mFramebufferTexture);
-		mFramebufferTexture = 0;
-		glDeleteRenderbuffersEXT(1, &mDepthRenderbufferObject);
-		mDepthRenderbufferObject = 0;
+		glDeleteFramebuffersEXT(1, &Framebuffer.mObject);
+		glDeleteTextures(1, &Framebuffer.mColorTexture);
+		glDeleteRenderbuffersEXT(1, &Framebuffer.mDepthRenderbuffer);
 	}
 #endif
+
+	Framebuffer.mObject = 0;
+	Framebuffer.mColorTexture = 0;
+	Framebuffer.mDepthRenderbuffer = 0;
+	Framebuffer.mWidth = 0;
+	Framebuffer.mHeight = 0;
 }
 
-QImage lcContext::GetRenderToTextureImage(int Width, int Height)
+void lcContext::BindFramebuffer(GLuint FramebufferObject)
 {
-	QImage Image(Width, Height, QImage::Format_ARGB32);
-	quint8* Buffer = Image.bits();
+	if (FramebufferObject == mFramebufferObject)
+		return;
+
+	if (gSupportsFramebufferObjectARB)
+		glBindFramebuffer(GL_FRAMEBUFFER, FramebufferObject);
+#ifndef LC_OPENGLES
+	else if (gSupportsFramebufferObjectEXT)
+		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, FramebufferObject);
+#endif
+
+	mFramebufferObject = FramebufferObject;
+}
+
+std::pair<lcFramebuffer, lcFramebuffer> lcContext::CreateRenderFramebuffer(int Width, int Height)
+{
+	if (gSupportsFramebufferObjectARB && QGLFormat::defaultFormat().samples() > 1)
+		return std::make_pair(CreateFramebuffer(Width, Height, true, true), CreateFramebuffer(Width, Height, false, false));
+	else
+		return std::make_pair(CreateFramebuffer(Width, Height, true, false), lcFramebuffer());
+}
+
+void lcContext::DestroyRenderFramebuffer(std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer)
+{
+	DestroyFramebuffer(RenderFramebuffer.first);
+	DestroyFramebuffer(RenderFramebuffer.second);
+}
+
+QImage lcContext::GetRenderFramebufferImage(const std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer)
+{
+	QImage Image(RenderFramebuffer.first.mWidth, RenderFramebuffer.first.mHeight, QImage::Format_ARGB32);
+
+	GetRenderFramebufferImage(RenderFramebuffer, Image.bits());
+
+	return Image;
+}
+
+void lcContext::GetRenderFramebufferImage(const std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer, quint8* Buffer)
+{
+	const int Width = RenderFramebuffer.first.mWidth;
+	const int Height = RenderFramebuffer.first.mHeight;
+	GLuint SavedFramebuffer = mFramebufferObject;
+
+	if (RenderFramebuffer.second.IsValid())
+	{
+#ifndef LC_OPENGLES
+		glBindFramebuffer(GL_DRAW_FRAMEBUFFER, RenderFramebuffer.second.mObject);
+		glBindFramebuffer(GL_READ_FRAMEBUFFER, RenderFramebuffer.first.mObject);
+		glBlitFramebuffer(0, 0, Width, Height, 0, 0, Width, Height, GL_COLOR_BUFFER_BIT, GL_LINEAR);
+		BindFramebuffer(RenderFramebuffer.second);
+#endif
+	}
+	else
+		BindFramebuffer(RenderFramebuffer.first);
 
 	glFinish();
 	glReadPixels(0, 0, Width, Height, GL_RGBA, GL_UNSIGNED_BYTE, Buffer);
+	BindFramebuffer(SavedFramebuffer);
 
 	for (int y = 0; y < (Height + 1) / 2; y++)
 	{
@@ -640,40 +747,16 @@ QImage lcContext::GetRenderToTextureImage(int Width, int Height)
 
 		for (int x = 0; x < Width; x++)
 		{
-			quint8 Red = Top[0];
-			quint8 Green = Top[1];
-			quint8 Blue = Top[2];
-			quint8 Alpha = Top[3];
+			QRgb TopColor = qRgba(Top[0], Top[1], Top[2], Top[3]);
+			QRgb BottomColor = qRgba(Bottom[0], Bottom[1], Bottom[2], Bottom[3]);
 
-			Top[0] = Bottom[2];
-			Top[1] = Bottom[1];
-			Top[2] = Bottom[0];
-			Top[3] = Bottom[3];
-
-			Bottom[0] = Blue;
-			Bottom[1] = Green;
-			Bottom[2] = Red;
-			Bottom[3] = Alpha;
+			*(QRgb*)Top = BottomColor;
+			*(QRgb*)Bottom = TopColor;
 
 			Top += 4;
 			Bottom += 4;
 		}
 	}
-
-	return Image;
-}
-
-bool lcContext::SaveRenderToTextureImage(const QString& FileName, int Width, int Height)
-{
-	QImage Image = GetRenderToTextureImage(Width, Height);
-    QImageWriter Writer(FileName);
-
-	bool Result = Writer.write(Image);
-
-	if (!Result)
-		QMessageBox::information(gMainWindow, tr("Error"), tr("Error writing to file '%1':\n%2").arg(FileName, Writer.errorString()));
-
-	return Result;
 }
 
 lcVertexBuffer lcContext::CreateVertexBuffer(int Size, const void* Data)
@@ -797,6 +880,7 @@ void lcContext::ClearVertexBuffer()
 	}
 	else
 	{
+#ifndef LC_OPENGLES
 		if (mNormalEnabled)
 			glDisableClientState(GL_NORMAL_ARRAY);
 
@@ -810,6 +894,7 @@ void lcContext::ClearVertexBuffer()
 		glNormalPointer(GL_BYTE, 0, nullptr);
 		glTexCoordPointer(2, GL_FLOAT, 0, nullptr);
 		glColorPointer(4, GL_FLOAT, 0, nullptr);
+#endif
 	}
 }
 
@@ -974,6 +1059,7 @@ void lcContext::SetVertexFormat(int BufferOffset, int PositionSize, int NormalSi
 	}
 	else
 	{
+#ifndef LC_OPENGLES
 		if (mVertexBufferOffset != VertexBufferPointer)
 		{
 			glVertexPointer(PositionSize, GL_FLOAT, VertexSize, VertexBufferPointer);
@@ -1033,6 +1119,7 @@ void lcContext::SetVertexFormat(int BufferOffset, int PositionSize, int NormalSi
 			glDisableClientState(GL_COLOR_ARRAY);
 			mColorEnabled = false;
 		}
+#endif
 	}
 }
 
