@@ -1,5 +1,4 @@
-#ifndef _LC_CONTEXT_H_
-#define _LC_CONTEXT_H_
+#pragma once
 
 #include "lc_array.h"
 #include "lc_math.h"
@@ -75,6 +74,30 @@ struct lcProgram
 	GLint EyePositionLocation;
 };
 
+class lcFramebuffer
+{
+public:
+	lcFramebuffer()
+	{
+	}
+
+	lcFramebuffer(int Width, int Height)
+		: mWidth(Width), mHeight(Height)
+	{
+	}
+
+	bool IsValid() const
+	{
+		return mObject != 0;
+	}
+
+	GLuint mObject = 0;
+	GLuint mColorTexture = 0;
+	GLuint mDepthRenderbuffer = 0;
+	int mWidth = 0;
+	int mHeight = 0;
+};
+
 class lcContext
 {
 public:
@@ -116,7 +139,8 @@ public:
 	void SetViewport(int x, int y, int Width, int Height);
 	void SetLineWidth(float LineWidth);
 	void SetSmoothShading(bool Smooth);
-	void BindTexture(GLuint Texture);
+	void BindTexture2D(GLuint Texture);
+	void BindTexture2DMS(GLuint Texture);
 
 	void SetColor(const lcVector4& Color)
 	{
@@ -130,10 +154,19 @@ public:
 	void SetEdgeColorIndex(int ColorIndex);
 	void SetInterfaceColor(lcInterfaceColor InterfaceColor);
 
-	bool BeginRenderToTexture(int Width, int Height);
-	void EndRenderToTexture();
-	QImage GetRenderToTextureImage(int Width, int Height);
-	bool SaveRenderToTextureImage(const QString& FileName, int Width, int Height);
+	void ClearFramebuffer();
+	lcFramebuffer CreateFramebuffer(int Width, int Height, bool Depth, bool Multisample);
+	void DestroyFramebuffer(lcFramebuffer& Framebuffer);
+	void BindFramebuffer(GLuint FramebufferObject);
+	void BindFramebuffer(const lcFramebuffer& Framebuffer)
+	{
+		BindFramebuffer(Framebuffer.mObject);
+	}
+
+	std::pair<lcFramebuffer, lcFramebuffer> CreateRenderFramebuffer(int Width, int Height);
+	void DestroyRenderFramebuffer(std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer);
+	QImage GetRenderFramebufferImage(const std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer);
+	void GetRenderFramebufferImage(const std::pair<lcFramebuffer, lcFramebuffer>& RenderFramebuffer, quint8* Buffer);
 
 	lcVertexBuffer CreateVertexBuffer(int Size, const void* Data);
 	void DestroyVertexBuffer(lcVertexBuffer& VertexBuffer);
@@ -170,7 +203,8 @@ protected:
 	bool mTexCoordEnabled;
 	bool mColorEnabled;
 
-	GLuint mTexture;
+	GLuint mTexture2D;
+	GLuint mTexture2DMS;
 	float mLineWidth;
 	int mMatrixMode;
 	bool mTextureEnabled;
@@ -187,12 +221,9 @@ protected:
 	bool mViewProjectionMatrixDirty;
 
 	GLuint mFramebufferObject;
-	GLuint mFramebufferTexture;
-	GLuint mDepthRenderbufferObject;
 
 	static lcProgram mPrograms[LC_NUM_MATERIALS];
 
 	Q_DECLARE_TR_FUNCTIONS(lcContext);
 };
 
-#endif // _LC_CONTEXT_H_
