@@ -28,6 +28,61 @@
 #define LC_LIBRARY_CACHE_ARCHIVE   0x0001
 #define LC_LIBRARY_CACHE_DIRECTORY 0x0002
 
+static lcVector2 lcCalculateTexCoord(const lcVector3& Position, const lcLibraryTextureMap* TextureMap)
+{
+	switch (TextureMap->Type)
+	{
+	case lcLibraryTextureMapType::PLANAR:
+		return lcVector2(lcDot3(Position, TextureMap->Params[0]) + TextureMap->Params[0].w, lcDot3(Position, TextureMap->Params[1]) + TextureMap->Params[1].w);
+
+	case lcLibraryTextureMapType::CYLINDRICAL:
+		{
+			const lcVector4& FrontPlane = TextureMap->Params[0];
+			const lcVector4& Plane1 = TextureMap->Params[2];
+			const lcVector4& Plane2 = TextureMap->Params[3];
+			lcVector2 TexCoord;
+
+			float DotPlane1 = lcDot(lcVector4(Position, 1.0f), Plane1);
+			lcVector3 PointInPlane1 = Position - lcVector3(Plane1) * DotPlane1;
+			float DotFrontPlane = lcDot(lcVector4(PointInPlane1, 1.0f), FrontPlane);
+			float DotPlane2 = lcDot(lcVector4(PointInPlane1, 1.0f), Plane2);
+
+			float Angle1 = atan2f(DotPlane2, DotFrontPlane) / LC_PI * TextureMap->Angle1;
+			TexCoord.x = lcClamp(0.5f + 0.5f * Angle1, 0.0f, 1.0f);
+
+			TexCoord.y = DotPlane1 / TextureMap->Params[1].w;
+
+			return TexCoord;
+		}
+
+	case lcLibraryTextureMapType::SPHERICAL:
+		{
+			const lcVector4& FrontPlane = TextureMap->Params[0];
+			const lcVector3& Center = (const lcVector3&)TextureMap->Params[1];
+			const lcVector4& Plane1 = TextureMap->Params[2];
+			const lcVector4& Plane2 = TextureMap->Params[3];
+			lcVector2 TexCoord;
+
+			lcVector3 VertexDir = Position - Center;
+			
+			float DotPlane1 = lcDot(lcVector4(Position, 1.0f), Plane1);
+			lcVector3 PointInPlane1 = Position - lcVector3(Plane1) * DotPlane1;
+			float DotFrontPlane = lcDot(lcVector4(PointInPlane1, 1.0f), FrontPlane);
+			float DotPlane2 = lcDot(lcVector4(PointInPlane1, 1.0f), Plane2);
+
+			float Angle1 = atan2f(DotPlane2, DotFrontPlane) / LC_PI * TextureMap->Angle1;
+			TexCoord.x = 0.5f + 0.5f * Angle1;
+
+			float Angle2 = asinf(DotPlane1 / lcLength(VertexDir)) / LC_PI * TextureMap->Angle2;
+			TexCoord.y = 0.5f - Angle2;
+
+			return TexCoord;
+		}
+	}
+
+	return lcVector2(0.0f, 0.0f);
+}
+
 lcPiecesLibrary::lcPiecesLibrary()
 	: mLoadMutex(QMutex::Recursive)
 {
@@ -679,8 +734,19 @@ void lcPiecesLibrary::ReadDirectoryDescriptions(const QFileInfoList (&FileLists)
 			{
 				const char* FileName = *(const char**)CachedDescription;
 				const char* Description = FileName + strlen(FileName) + 1;
-				strcpy(Info->m_strDescription, Description);
-				return;
+				uint64_t CachedFileTime = *(uint64_t*)(Description + strlen(Description) + 1 + 4 + 1);
+
+#if (QT_VERSION >= QT_VERSION_CHECK(4, 7, 0))
+				quint64 FileTime = FileLists[Info->mFolderType][Info->mFolderIndex].lastModified().toMSecsSinceEpoch();
+#else
+				quint64 FileTime = FileLists[Info->mFolderType][Info->mFolderIndex].lastModified().toTime_t();
+#endif
+
+				if (FileTime == CachedFileTime)
+				{
+					strcpy(Info->m_strDescription, Description);
+					return;
+				}
 			}
 		}
 
@@ -777,7 +843,7 @@ void lcPiecesLibrary::ReadDirectoryDescriptions(const QFileInfoList (&FileLists)
 #else
 			quint64 FileTime = FileLists[Info->mFolderType][Info->mFolderIndex].lastModified().toTime_t();
 #endif
-		
+
 			IndexFile.WriteU64(FileTime);
 		}
 
@@ -1368,10 +1434,19 @@ lcMesh* lcPiecesLibrary::CreateMesh(PieceInfo* Info, lcLibraryMeshData& MeshData
 
 			DstVertex.Position = lcVector3LDrawToLeoCAD(SrcVertex.Position);
 			DstVertex.Normal = lcPackNormal(lcVector3LDrawToLeoCAD(SrcVertex.Normal));
+		}
 
-			lcVector3& Position = DstVertex.Position;
-			Min = lcMin(Min, Position);
-			Max = lcMax(Max, Position);
+		for (const lcLibraryMeshSection* Section : MeshData.mSections[MeshDataIdx])
+		{
+			if (Section->mPrimitiveType != LC_MESH_TRIANGLES)
+				continue;
+
+			for (quint32 Index : Section->mIndices)
+			{
+				lcVector3 Position = lcVector3LDrawToLeoCAD(Vertices[Index].Position);
+				Min = lcMin(Min, Position);
+				Max = lcMax(Max, Position);
+			}
 		}
 	}
 
@@ -1643,37 +1718,44 @@ void lcPiecesLibrary::UnloadUnusedParts()
 
 bool lcPiecesLibrary::LoadTexture(lcTexture* Texture)
 {
-	char Name[LC_MAXPATH], FileName[LC_MAXPATH];
-
-	strcpy(Name, Texture->mName);
-	strlwr(Name);
+	char FileName[LC_MAXPATH];
 
 	if (mZipFiles[LC_ZIPFILE_OFFICIAL])
 	{
 		lcMemFile TextureFile;
 
-		sprintf(FileName, "parts/textures/%s.png", Name);
+		sprintf(FileName, "parts/textures/%s.png", Texture->mName);
 
 		if (!mZipFiles[LC_ZIPFILE_UNOFFICIAL] || !mZipFiles[LC_ZIPFILE_UNOFFICIAL]->ExtractFile(FileName, TextureFile))
 		{
-			sprintf(FileName, "ldraw/parts/textures/%s.png", Name);
+			sprintf(FileName, "ldraw/parts/textures/%s.png", Texture->mName);
 
 			if (!mZipFiles[LC_ZIPFILE_OFFICIAL]->ExtractFile(FileName, TextureFile))
 				return false;
 		}
 
-		if (!Texture->Load(TextureFile))
-			return false;
+		return Texture->Load(TextureFile);
 	}
 	else
 	{
+		sprintf(FileName, "parts/textures/%s.png", Texture->mName);
+
+		if (Texture->Load(mLibraryDir.absoluteFilePath(QLatin1String(FileName))))
+			return true;
+
+#if defined(Q_OS_MACOS) || defined(Q_OS_LINUX)
+		char Name[LC_MAXPATH];
+
+		strcpy(Name, Texture->mName);
+		strlwr(Name);
+
 		sprintf(FileName, "parts/textures/%s.png", Name);
 
-		if (!Texture->Load(mLibraryDir.absoluteFilePath(QLatin1String(FileName))))
-			return false;
+		return Texture->Load(mLibraryDir.absoluteFilePath(QLatin1String(FileName)));
+#else
+		return false;
+#endif
 	}
-
-	return true;
 }
 
 void lcPiecesLibrary::ReleaseTexture(lcTexture* Texture)
@@ -1873,15 +1955,8 @@ bool lcPiecesLibrary::ReadMeshData(lcFile& File, const lcMatrix44& CurrentTransf
 						End++;
 					*End = 0;
 
-					if (!strcmp(Token, "PLANAR"))
+					auto CleanTextureName = [](char* FileName)
 					{
-						Token += 7;
-
-						char FileName[LC_MAXPATH];
-						lcVector3 Points[3];
-
-						sscanf(Token, "%f %f %f %f %f %f %f %f %f %s", &Points[0].x, &Points[0].y, &Points[0].z, &Points[1].x, &Points[1].y, &Points[1].z, &Points[2].x, &Points[2].y, &Points[2].z, FileName);
-
 						char* Ch;
 						for (Ch = FileName; *Ch; Ch++)
 						{
@@ -1897,11 +1972,24 @@ bool lcPiecesLibrary::ReadMeshData(lcFile& File, const lcMatrix44& CurrentTransf
 							if (!memcmp(Ch, ".PNG", 4))
 								*Ch = 0;
 						}
+					};
+
+					if (!strcmp(Token, "PLANAR"))
+					{
+						Token += 7;
+
+						char FileName[LC_MAXPATH];
+						lcVector3 Points[3];
+
+						sscanf(Token, "%f %f %f %f %f %f %f %f %f %s", &Points[0].x, &Points[0].y, &Points[0].z, &Points[1].x, &Points[1].y, &Points[1].z, &Points[2].x, &Points[2].y, &Points[2].z, FileName);
+
+						CleanTextureName(FileName);
 
 						lcLibraryTextureMap& Map = TextureStack.Add();
 						Map.Next = false;
 						Map.Fallback = false;
 						Map.Texture = FindTexture(FileName, CurrentProject, SearchProjectFolder);
+						Map.Type = lcLibraryTextureMapType::PLANAR;
 
 						for (int EdgeIdx = 0; EdgeIdx < 2; EdgeIdx++)
 						{
@@ -1915,6 +2003,63 @@ bool lcPiecesLibrary::ReadMeshData(lcFile& File, const lcMatrix44& CurrentTransf
 							Map.Params[EdgeIdx].w = -lcDot(Normal, Points[0]) / Length;
 						}
 					}
+					else if (!strcmp(Token, "CYLINDRICAL"))
+					{
+						Token += 12;
+
+						char FileName[LC_MAXPATH];
+						lcVector3 Points[3];
+						float Angle;
+
+						sscanf(Token, "%f %f %f %f %f %f %f %f %f %f %s", &Points[0].x, &Points[0].y, &Points[0].z, &Points[1].x, &Points[1].y, &Points[1].z, &Points[2].x, &Points[2].y, &Points[2].z, &Angle, FileName);
+
+						CleanTextureName(FileName);
+
+						lcLibraryTextureMap& Map = TextureStack.Add();
+						Map.Next = false;
+						Map.Fallback = false;
+						Map.Texture = FindTexture(FileName, CurrentProject, SearchProjectFolder);
+
+						Map.Type = lcLibraryTextureMapType::CYLINDRICAL;
+						lcVector3 Up = Points[0] - Points[1];
+						float UpLength = lcLength(Up);
+						lcVector3 Front = lcNormalize(Points[2] - Points[1]);
+						lcVector3 Plane1Normal = Up / UpLength;
+						lcVector3 Plane2Normal = lcNormalize(lcCross(Up, Front));
+						Map.Params[0] = lcVector4(Front, -lcDot(Front, Points[1]));
+						Map.Params[1] = lcVector4(Points[1], UpLength);
+						Map.Params[2] = lcVector4(Plane1Normal, -lcDot(Plane1Normal, Points[1]));
+						Map.Params[3] = lcVector4(Plane2Normal, -lcDot(Plane2Normal, Points[1]));
+						Map.Angle1 = 360.0f / Angle;
+					}
+					else if (!strcmp(Token, "SPHERICAL"))
+					{
+						Token += 10;
+
+						char FileName[LC_MAXPATH];
+						lcVector3 Points[3];
+						float Angle1, Angle2;
+
+						sscanf(Token, "%f %f %f %f %f %f %f %f %f %f %f %s", &Points[0].x, &Points[0].y, &Points[0].z, &Points[1].x, &Points[1].y, &Points[1].z, &Points[2].x, &Points[2].y, &Points[2].z, &Angle1, &Angle2, FileName);
+
+						CleanTextureName(FileName);
+
+						lcLibraryTextureMap& Map = TextureStack.Add();
+						Map.Next = false;
+						Map.Fallback = false;
+						Map.Texture = FindTexture(FileName, CurrentProject, SearchProjectFolder);
+						Map.Type = lcLibraryTextureMapType::SPHERICAL;
+
+						lcVector3 Front = lcNormalize(Points[1] - Points[0]);
+						lcVector3 Plane1Normal = lcNormalize(lcCross(Front, Points[2] - Points[0]));
+						lcVector3 Plane2Normal = lcNormalize(lcCross(Plane1Normal, Front));
+						Map.Params[0] = lcVector4(Front, -lcDot(Front, Points[0]));
+						Map.Params[1] = lcVector4(Points[0], 0.0f);
+						Map.Params[2] = lcVector4(Plane1Normal, -lcDot(Plane1Normal, Points[0]));
+						Map.Params[3] = lcVector4(Plane2Normal, -lcDot(Plane2Normal, Points[0]));
+						Map.Angle1 = 360.0f / Angle1;
+						Map.Angle2 = 180.0f / Angle2;
+					}
 				}
 				else if (!strcmp(Token, "FALLBACK"))
 				{
@@ -2017,7 +2162,7 @@ bool lcPiecesLibrary::ReadMeshData(lcFile& File, const lcMatrix44& CurrentTransf
 						*Ch = '/';
 				}
 
-				lcLibraryPrimitive* Primitive = FindPrimitive(FileName);
+				lcLibraryPrimitive* Primitive = !TextureMap ? FindPrimitive(FileName) : nullptr;
 				lcMatrix44 IncludeTransform(lcVector4(fm[3], fm[6], fm[9], 0.0f), lcVector4(fm[4], fm[7], fm[10], 0.0f), lcVector4(fm[5], fm[8], fm[11], 0.0f), lcVector4(fm[0], fm[1], fm[2], 1.0f));
 				IncludeTransform = lcMul(IncludeTransform, CurrentTransform);
 				bool Mirror = IncludeTransform.Determinant() < 0.0f;
@@ -2118,6 +2263,82 @@ bool lcPiecesLibrary::ReadMeshData(lcFile& File, const lcMatrix44& CurrentTransf
 
 						break;
 					}
+					else
+					{
+						bool Found = false;
+
+						if (mZipFiles[LC_ZIPFILE_OFFICIAL])
+						{
+							lcMemFile IncludeFile;
+
+							auto LoadIncludeFile = [&IncludeFile, &FileName, this](const char* Folder, int ZipFileIndex)
+							{
+								char IncludeFileName[LC_MAXPATH];
+								sprintf(IncludeFileName, Folder, FileName);
+								return mZipFiles[ZipFileIndex]->ExtractFile(IncludeFileName, IncludeFile);
+							};
+
+							if (mHasUnofficial)
+							{
+								Found = LoadIncludeFile("parts/%s", LC_ZIPFILE_UNOFFICIAL);
+
+								if (!Found)
+									Found = LoadIncludeFile("p/%s", LC_ZIPFILE_UNOFFICIAL);
+							}
+
+							if (!Found)
+							{
+								Found = LoadIncludeFile("ldraw/parts/%s", LC_ZIPFILE_OFFICIAL);
+
+								if (!Found)
+									Found = LoadIncludeFile("ldraw/p/%s", LC_ZIPFILE_OFFICIAL);
+							}
+
+							if (Found)
+								ReadMeshData(IncludeFile, IncludeTransform, ColorCode, Mirror ^ InvertNext, TextureStack, MeshData, MeshDataType, Optimize, CurrentProject, SearchProjectFolder);
+						}
+						else
+						{
+							lcDiskFile IncludeFile;
+
+							auto LoadIncludeFile = [&IncludeFile, &FileName, this](const char* Folder)
+							{
+								char IncludeFileName[LC_MAXPATH];
+								sprintf(IncludeFileName, Folder, FileName);
+								IncludeFile.SetFileName(mLibraryDir.absoluteFilePath(QLatin1String(IncludeFileName)));
+								if (IncludeFile.Open(QIODevice::ReadOnly))
+									return true;
+
+#if defined(Q_OS_MACOS) || defined(Q_OS_LINUX)
+								// todo: instead of using strlwr, search the parts/primitive lists and get the file name from there
+								strlwr(IncludeFileName);
+								IncludeFile.SetFileName(mLibraryDir.absoluteFilePath(QLatin1String(IncludeFileName)));
+								return IncludeFile.Open(QIODevice::ReadOnly);
+#else
+								return false;
+#endif
+							};
+
+							if (mHasUnofficial)
+							{
+								Found = LoadIncludeFile("unofficial/parts/%s");
+
+								if (!Found)
+									Found = LoadIncludeFile("unofficial/p/%s");
+							}
+
+							if (!Found)
+							{
+								Found = LoadIncludeFile("parts/%s");
+
+								if (!Found)
+									Found = LoadIncludeFile("p/%s");
+							}
+
+							if (Found)
+								ReadMeshData(IncludeFile, IncludeTransform, ColorCode, Mirror ^ InvertNext, TextureStack, MeshData, MeshDataType, Optimize, CurrentProject, SearchProjectFolder);
+						}
+					}
 				}
 			} break;
 
@@ -2265,6 +2486,20 @@ void lcLibraryMeshData::AddVertices(lcMeshDataType MeshDataType, int VertexCount
 	*VertexBuffer = &Vertices[CurrentSize];
 }
 
+const float lcDistanceEpsilon = 0.01f; // Maximum value for 50591.dat
+const float lcTexCoordEpsilon = 0.01f;
+
+inline bool lcCompareVertices(const lcVector3& Position1, const lcVector3& Position2)
+{
+	return fabsf(Position1.x - Position2.x) < lcDistanceEpsilon && fabsf(Position1.y - Position2.y) < lcDistanceEpsilon && fabsf(Position1.z - Position2.z) < lcDistanceEpsilon;
+}
+
+inline bool lcCompareVertices(const lcVector3& Position1, const lcVector2& TexCoord1, const lcVector3& Position2, const lcVector2& TexCoord2)
+{
+	return fabsf(Position1.x - Position2.x) < lcDistanceEpsilon && fabsf(Position1.y - Position2.y) < lcDistanceEpsilon && fabsf(Position1.z - Position2.z) < lcDistanceEpsilon &&
+	       fabsf(TexCoord1.x - TexCoord2.x) < lcTexCoordEpsilon && fabsf(TexCoord1.y - TexCoord2.y) < lcTexCoordEpsilon;
+}
+
 quint32 lcLibraryMeshData::AddVertex(lcMeshDataType MeshDataType, const lcVector3& Position, bool Optimize)
 {
 	lcArray<lcLibraryMeshVertex>& VertexArray = mVertices[MeshDataType];
@@ -2275,7 +2510,7 @@ quint32 lcLibraryMeshData::AddVertex(lcMeshDataType MeshDataType, const lcVector
 		{
 			lcLibraryMeshVertex& Vertex = VertexArray[VertexIdx];
 
-			if (Position == Vertex.Position)
+			if (lcCompareVertices(Position, Vertex.Position))
 				return VertexIdx;
 		}
 	}
@@ -2288,8 +2523,6 @@ quint32 lcLibraryMeshData::AddVertex(lcMeshDataType MeshDataType, const lcVector
 	return VertexArray.GetSize() - 1;
 }
 
-const float DistanceEpsilon = 0.05f;
-
 quint32 lcLibraryMeshData::AddVertex(lcMeshDataType MeshDataType, const lcVector3& Position, const lcVector3& Normal, bool Optimize)
 {
 	lcArray<lcLibraryMeshVertex>& VertexArray = mVertices[MeshDataType];
@@ -2300,8 +2533,7 @@ quint32 lcLibraryMeshData::AddVertex(lcMeshDataType MeshDataType, const lcVector
 		{
 			lcLibraryMeshVertex& Vertex = VertexArray[VertexIdx];
 
-			if (fabsf(Position.x - Vertex.Position.x) < DistanceEpsilon && fabsf(Position.y - Vertex.Position.y) < DistanceEpsilon && fabsf(Position.z - Vertex.Position.z) < DistanceEpsilon)
-//			if (Position == Vertex.Position)
+			if (lcCompareVertices(Position, Vertex.Position))
 			{
 				if (Vertex.NormalWeight == 0.0f)
 				{
@@ -2337,7 +2569,7 @@ quint32 lcLibraryMeshData::AddTexturedVertex(lcMeshDataType MeshDataType, const
 		{
 			lcLibraryMeshVertexTextured& Vertex = VertexArray[VertexIdx];
 
-			if (Position == Vertex.Position && TexCoord == Vertex.TexCoord)
+			if (lcCompareVertices(Position, TexCoord, Vertex.Position, Vertex.TexCoord))
 				return VertexIdx;
 		}
 	}
@@ -2361,7 +2593,7 @@ quint32 lcLibraryMeshData::AddTexturedVertex(lcMeshDataType MeshDataType, const
 		{
 			lcLibraryMeshVertexTextured& Vertex = VertexArray[VertexIdx];
 
-			if (Position == Vertex.Position && TexCoord == Vertex.TexCoord)
+			if (lcCompareVertices(Position, TexCoord, Vertex.Position, Vertex.TexCoord))
 			{
 				if (Vertex.NormalWeight == 0.0f)
 				{
@@ -2490,10 +2722,10 @@ void lcLibraryMeshData::AddLine(lcMeshDataType MeshDataType, int LineType, quint
 	}
 }
 
-void lcLibraryMeshData::AddTexturedLine(lcMeshDataType MeshDataType, int LineType, quint32 ColorCode, bool WindingCCW, const lcLibraryTextureMap& Map, const lcVector3* Vertices, bool Optimize)
+void lcLibraryMeshData::AddTexturedLine(lcMeshDataType MeshDataType, int LineType, quint32 ColorCode, bool WindingCCW, const lcLibraryTextureMap& TextureMap, const lcVector3* Vertices, bool Optimize)
 {
 	lcMeshPrimitiveType PrimitiveType = (LineType == 2) ? LC_MESH_TEXTURED_LINES : LC_MESH_TEXTURED_TRIANGLES;
-	lcLibraryMeshSection* Section = AddSection(MeshDataType, PrimitiveType, ColorCode, Map.Texture);
+	lcLibraryMeshSection* Section = AddSection(MeshDataType, PrimitiveType, ColorCode, TextureMap.Texture);
 
 	int QuadIndices[4] = { 0, 1, 2, 3 };
 	int Indices[4] = { -1, -1, -1, -1 };
@@ -2508,12 +2740,125 @@ void lcLibraryMeshData::AddTexturedLine(lcMeshDataType MeshDataType, int LineTyp
 		if (!WindingCCW)
 			Normal = -Normal;
 
+		lcVector2 TexCoords[4];
+
 		for (int IndexIdx = 0; IndexIdx < lcMin(LineType, 4); IndexIdx++)
 		{
 			const lcVector3& Position = Vertices[QuadIndices[IndexIdx]];
-			lcVector2 TexCoord(lcDot3(lcVector3(Position.x, Position.y, Position.z), Map.Params[0]) + Map.Params[0].w,
-							   lcDot3(lcVector3(Position.x, Position.y, Position.z), Map.Params[1]) + Map.Params[1].w);
-			Indices[IndexIdx] = AddTexturedVertex(MeshDataType, Position, Normal, TexCoord, Optimize);
+			TexCoords[QuadIndices[IndexIdx]] = lcCalculateTexCoord(Position, &TextureMap);
+		}
+
+		if (TextureMap.Type == lcLibraryTextureMapType::CYLINDRICAL || TextureMap.Type == lcLibraryTextureMapType::SPHERICAL)
+		{
+			auto CheckTexCoordsWrap = [&TexCoords, &Vertices, &TextureMap](int Index1, int Index2, int Index3)
+			{
+				float u12 = fabsf(TexCoords[Index1].x - TexCoords[Index2].x);
+				float u13 = fabsf(TexCoords[Index1].x - TexCoords[Index3].x);
+				float u23 = fabsf(TexCoords[Index2].x - TexCoords[Index3].x);
+
+				if (u12 < 0.5f && u13 < 0.5f && u23 < 0.5f)
+					return;
+
+				const lcVector4& Plane2 = TextureMap.Params[3];
+				float Dot1 = fabsf(lcDot(Plane2, lcVector4(Vertices[Index1], 1.0f)));
+				float Dot2 = fabsf(lcDot(Plane2, lcVector4(Vertices[Index2], 1.0f)));
+				float Dot3 = fabsf(lcDot(Plane2, lcVector4(Vertices[Index3], 1.0f)));
+
+				if (Dot1 > Dot2)
+				{
+					if (Dot1 > Dot3)
+					{
+						if (u12 > 0.5f)
+							TexCoords[Index2].x += TexCoords[Index2].x < 0.5f ? 1.0f : -1.0f;
+
+						if (u13 > 0.5f)
+							TexCoords[Index3].x += TexCoords[Index3].x < 0.5f ? 1.0f : -1.0f;
+					}
+					else
+					{
+						if (u13 > 0.5f)
+							TexCoords[Index1].x += TexCoords[Index1].x < 0.5f ? 1.0f : -1.0f;
+
+						if (u23 > 0.5f)
+							TexCoords[Index2].x += TexCoords[Index2].x < 0.5f ? 1.0f : -1.0f;
+					}
+				}
+				else
+				{
+					if (Dot2 > Dot3)
+					{
+						if (u12 > 0.5f)
+							TexCoords[Index1].x += TexCoords[Index1].x < 0.5f ? 1.0f : -1.0f;
+
+						if (u23 > 0.5f)
+							TexCoords[Index3].x += TexCoords[Index3].x < 0.5f ? 1.0f : -1.0f;
+					}
+					else
+					{
+						if (u13 > 0.5f)
+							TexCoords[Index1].x += TexCoords[Index1].x < 0.5f ? 1.0f : -1.0f;
+
+						if (u23 > 0.5f)
+							TexCoords[Index2].x += TexCoords[Index2].x < 0.5f ? 1.0f : -1.0f;
+					}
+				}
+			};
+
+			CheckTexCoordsWrap(QuadIndices[0], QuadIndices[1], QuadIndices[2]);
+
+			if (LineType == 4)
+				CheckTexCoordsWrap(QuadIndices[2], QuadIndices[3], QuadIndices[0]);
+		}
+
+		if (TextureMap.Type == lcLibraryTextureMapType::SPHERICAL)
+		{
+			auto CheckTexCoordsPole = [&TexCoords, &Vertices, &TextureMap](int Index1, int Index2, int Index3)
+			{
+				const lcVector4& FrontPlane = TextureMap.Params[0];
+				const lcVector4& Plane2 = TextureMap.Params[3];
+				int PoleIndex;
+				int EdgeIndex1, EdgeIndex2;
+
+				if (fabsf(lcDot(lcVector4(Vertices[Index1], 1.0f), FrontPlane)) < 0.01f && fabsf(lcDot(lcVector4(Vertices[Index1], 1.0f), Plane2)) < 0.01f)
+				{
+					PoleIndex = Index1;
+					EdgeIndex1 = Index2;
+					EdgeIndex2 = Index3;
+				}
+				else if (fabsf(lcDot(lcVector4(Vertices[Index2], 1.0f), FrontPlane)) < 0.01f && fabsf(lcDot(lcVector4(Vertices[Index2], 1.0f), Plane2)) < 0.01f)
+				{
+					PoleIndex = Index2;
+					EdgeIndex1 = Index1;
+					EdgeIndex2 = Index3;
+				}
+				else if (fabsf(lcDot(lcVector4(Vertices[Index3], 1.0f), FrontPlane)) < 0.01f && fabsf(lcDot(lcVector4(Vertices[Index3], 1.0f), Plane2)) < 0.01f)
+				{
+					PoleIndex = Index3;
+					EdgeIndex1 = Index1;
+					EdgeIndex2 = Index2;
+				}
+				else
+					return;
+
+				lcVector3 OppositeEdge = Vertices[EdgeIndex2] - Vertices[EdgeIndex1];
+				lcVector3 SideEdge = Vertices[PoleIndex] - Vertices[EdgeIndex1];
+
+				float OppositeLength = lcLength(OppositeEdge);
+				float Projection = lcDot(OppositeEdge, SideEdge) / (OppositeLength * OppositeLength);
+
+				TexCoords[PoleIndex].x = TexCoords[EdgeIndex1].x + (TexCoords[EdgeIndex2].x - TexCoords[EdgeIndex1].x) * Projection;
+			};
+
+			CheckTexCoordsPole(QuadIndices[0], QuadIndices[1], QuadIndices[2]);
+
+			if (LineType == 4)
+				CheckTexCoordsPole(QuadIndices[2], QuadIndices[3], QuadIndices[0]);
+		}
+
+		for (int IndexIdx = 0; IndexIdx < lcMin(LineType, 4); IndexIdx++)
+		{
+			const lcVector3& Position = Vertices[QuadIndices[IndexIdx]];
+			Indices[IndexIdx] = AddTexturedVertex(MeshDataType, Position, Normal, TexCoords[QuadIndices[IndexIdx]], Optimize);
 		}
 	}
 	else
@@ -2521,8 +2866,7 @@ void lcLibraryMeshData::AddTexturedLine(lcMeshDataType MeshDataType, int LineTyp
 		for (int IndexIdx = 0; IndexIdx < lcMin(LineType, 4); IndexIdx++)
 		{
 			const lcVector3& Position = Vertices[QuadIndices[IndexIdx]];
-			lcVector2 TexCoord(lcDot3(lcVector3(Position.x, Position.y, Position.z), Map.Params[0]) + Map.Params[0].w,
-							   lcDot3(lcVector3(Position.x, Position.y, Position.z), Map.Params[1]) + Map.Params[1].w);
+			lcVector2 TexCoord = lcCalculateTexCoord(Position, &TextureMap);
 			Indices[IndexIdx] = AddTexturedVertex(MeshDataType, Position, TexCoord, Optimize);
 		}
 	}
@@ -2616,8 +2960,7 @@ void lcLibraryMeshData::AddMeshData(const lcLibraryMeshData& Data, const lcMatri
 			{
 				const lcLibraryMeshVertex& SrcVertex = DataVertices[SrcVertexIdx];
 				lcVector3 Position = lcMul31(SrcVertex.Position, Transform);
-				lcVector2 TexCoord(lcDot3(lcVector3(Position.x, Position.y, Position.z), TextureMap->Params[0]) + TextureMap->Params[0].w,
-								   lcDot3(lcVector3(Position.x, Position.y, Position.z), TextureMap->Params[1]) + TextureMap->Params[1].w);
+				lcVector2 TexCoord = lcCalculateTexCoord(Position, TextureMap);
 				int Index;
 
 				if (DataVertices[SrcVertexIdx].NormalWeight == 0.0f)
@@ -2785,8 +3128,7 @@ void lcLibraryMeshData::AddMeshDataNoDuplicateCheck(const lcLibraryMeshData& Dat
 				lcLibraryMeshVertexTextured& DstVertex = TexturedVertices.Add();
 
 				lcVector3 Position = lcMul31(SrcVertex.Position, Transform);
-				lcVector2 TexCoord(lcDot3(lcVector3(Position.x, Position.y, Position.z), TextureMap->Params[0]) + TextureMap->Params[0].w,
-								   lcDot3(lcVector3(Position.x, Position.y, Position.z), TextureMap->Params[1]) + TextureMap->Params[1].w);
+				lcVector2 TexCoord = lcCalculateTexCoord(Position, TextureMap);
 
 				DstVertex.Position = Position;
 				DstVertex.Normal = lcNormalize(lcMul30(SrcVertex.Normal, Transform));
