@@ -58,10 +58,20 @@ public:
 	lcArray<quint32> mIndices;
 };
 
+enum class lcLibraryTextureMapType
+{
+	PLANAR,
+	CYLINDRICAL,
+	SPHERICAL
+};
+
 struct lcLibraryTextureMap
 {
-	lcVector4 Params[2];
 	lcTexture* Texture;
+	lcVector4 Params[4];
+	float Angle1;
+	float Angle2;
+	lcLibraryTextureMapType Type;
 	bool Fallback;
 	bool Next;
 };
