@@ -1,5 +1,4 @@
-#ifndef _LC_MAINWINDOW_H_
-#define _LC_MAINWINDOW_H_
+#pragma once
 
 #include "lc_basewindow.h"
 #include "lc_array.h"
@@ -22,6 +21,7 @@ class QPrinter;
 
 struct lcSearchOptions
 {
+	bool SearchValid;
 	bool MatchInfo;
 	bool MatchColor;
 	bool MatchName;
@@ -30,6 +30,19 @@ struct lcSearchOptions
 	char Name[256];
 };
 
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
+typedef QTabWidget lcTabWidget;
+#else
+class lcTabWidget : public QTabWidget
+{
+public:
+	QTabBar* tabBar()
+	{
+		return QTabWidget::tabBar();
+	}
+};
+#endif
+
 class lcModelTabWidget : public QWidget
 {
 	Q_OBJECT
@@ -135,9 +148,9 @@ public:
 		return mMoveSnapEnabled ? SnapZTable[mMoveZSnapIndex] : 0.0f;
 	}
 
-	int GetAngleSnap() const
+	float GetAngleSnap() const
 	{
-		const int AngleTable[] = { 0, 1, 5, 10, 15, 30, 45, 60, 90, 180 };
+		const float AngleTable[] = { 0.0f, 1.0f, 5.0f, 15.0f, 22.5f, 30.0f, 45.0f, 60.0f, 90.0f, 180.0f };
 		return mAngleSnapEnabled ? AngleTable[mAngleSnapIndex] : 0.0f;
 	}
 
@@ -158,26 +171,16 @@ public:
 		return mAngleSnapEnabled ? QString::number(GetAngleSnap()) : tr("None");
 	}
 
-	bool GetLockX() const
-	{
-		return mLockX;
-	}
-
-	bool GetLockY() const
-	{
-		return mLockY;
-	}
-
-	bool GetLockZ() const
-	{
-		return mLockZ;
-	}
-
 	bool GetRelativeTransform() const
 	{
 		return mRelativeTransform;
 	}
 
+	lcSelectionMode GetSelectionMode() const
+	{
+		return mSelectionMode;
+	}
+
 	PieceInfo* GetCurrentPieceInfo() const
 	{
 		return mCurrentPieceInfo;
@@ -214,9 +217,9 @@ public:
 		return mPartSelectionWidget;
 	}
 
-	QMenu* GetCameraMenu() const
+	QMenu* GetToolsMenu() const
 	{
-		return mCameraMenu;
+		return mToolsMenu;
 	}
 
 	QMenu* GetViewpointMenu() const
@@ -224,8 +227,23 @@ public:
 		return mViewpointMenu;
 	}
 
-	bool DoDialog(LC_DIALOG_TYPE Type, void* Data);
+	QMenu* GetCameraMenu() const
+	{
+		return mCameraMenu;
+	}
 
+	QMenu* GetProjectionMenu() const
+	{
+		return mProjectionMenu;
+	}
+
+	QMenu* GetShadingMenu() const
+	{
+		return mShadingMenu;
+	}
+
+	QByteArray GetTabLayout();
+	void RestoreTabLayout(const QByteArray& TabLayout);
 	void RemoveAllModelTabs();
 	void SetCurrentModelTab(lcModel* Model);
 	void ResetCameras();
@@ -242,16 +260,16 @@ public:
 	void SetMoveXYSnapIndex(int Index);
 	void SetMoveZSnapIndex(int Index);
 	void SetAngleSnapIndex(int Index);
-	void SetLockX(bool LockX);
-	void SetLockY(bool LockY);
-	void SetLockZ(bool LockZ);
 	void SetRelativeTransform(bool RelativeTransform);
 	void SetCurrentPieceInfo(PieceInfo* Info);
+	void SetShadingMode(lcShadingMode ShadingMode);
+	void SetSelectionMode(lcSelectionMode SelectionMode);
 
 	void NewProject();
 	bool OpenProject(const QString& FileName);
 	void MergeProject();
 	void ImportLDD();
+	void ImportInventory();
 	bool SaveProject(const QString& FileName);
 	bool SaveProjectIfModified();
 	bool SetModelFromFocus();
@@ -281,6 +299,8 @@ public:
 	void UpdateCurrentCamera(int CameraIndex);
 	void UpdatePerspective();
 	void UpdateCameraMenu();
+	void UpdateShadingMode();
+	void UpdateSelectionMode();
 	void UpdateModels();
 	void UpdateCategories();
 	void UpdateTitle();
@@ -295,7 +315,13 @@ public:
 	lcSearchOptions mSearchOptions;
 	QAction* mActions[LC_NUM_COMMANDS];
 
+public slots:
+	void ProjectFileChanged(const QString& Path);
+
 protected slots:
+	void ModelTabContextMenuRequested(const QPoint& Point);
+	void ModelTabCloseOtherTabs();
+	void ModelTabResetViews();
 	void ModelTabClosed(int Index);
 	void ModelTabChanged(int Index);
 	void ClipboardChanged();
@@ -314,8 +340,11 @@ protected:
 	void CreateToolBars();
 	void CreateStatusBar();
 	void SplitView(Qt::Orientation Orientation);
+	void ShowSearchDialog();
 	void ShowUpdatesDialog();
 	void ShowAboutDialog();
+	void ShowHTMLDialog();
+	void ShowRenderDialog();
 	void ShowPrintDialog();
 
 	lcModelTabWidget* GetTabWidgetForModel(lcModel* Model) const
@@ -339,15 +368,14 @@ protected:
 	int mMoveXYSnapIndex;
 	int mMoveZSnapIndex;
 	int mAngleSnapIndex;
-	bool mLockX;
-	bool mLockY;
-	bool mLockZ;
 	bool mRelativeTransform;
 	PieceInfo* mCurrentPieceInfo;
+	lcSelectionMode mSelectionMode;
+	int mModelTabWidgetContextMenuIndex;
 
 	QAction* mActionFileRecentSeparator;
 
-	QTabWidget* mModelTabWidget;
+	lcTabWidget* mModelTabWidget;
 	QToolBar* mStandardToolBar;
 	QToolBar* mToolsToolBar;
 	QToolBar* mTimeToolBar;
@@ -369,10 +397,12 @@ protected:
 	QLabel* mStatusSnapLabel;
 	QLabel* mStatusTimeLabel;
 
-	QMenu* mCameraMenu;
+	QMenu* mToolsMenu;
 	QMenu* mViewpointMenu;
+	QMenu* mCameraMenu;
+	QMenu* mProjectionMenu;
+	QMenu* mShadingMenu;
+	QMenu* mSelectionModeMenu;
 };
 
 extern class lcMainWindow* gMainWindow;
-
-#endif // _LC_MAINWND_H_
