@@ -86,6 +86,8 @@ public:
 	{
 	}
 
+	explicit lcVector3(const lcVector4& v);
+
 	operator const float*() const
 	{
 		return (const float*)this;
@@ -252,6 +254,11 @@ public:
 	lcVector4 r[4];
 };
 
+inline lcVector3::lcVector3(const lcVector4& v)
+	: x(v.x), y(v.y), z(v.z)
+{
+}
+
 inline lcVector3 operator+(const lcVector3& a, const lcVector3& b)
 {
 	return lcVector3(a.x + b.x, a.y + b.y, a.z + b.z);
@@ -363,6 +370,13 @@ inline bool operator!=(const lcVector3& a, const lcVector3& b)
 
 #ifndef QT_NO_DEBUG
 
+inline QDebug operator<<(QDebug Debug, const lcVector2& v)
+{
+	QDebugStateSaver Saver(Debug);
+	Debug.nospace() << '(' << v.x << ", " << v.y << ')';
+	return Debug;
+}
+
 inline QDebug operator<<(QDebug Debug, const lcVector3& v)
 {
 	QDebugStateSaver Saver(Debug);
@@ -1615,9 +1629,7 @@ inline void lcClosestPointsBetweenLines(const lcVector3& Line1a, const lcVector3
 	}
 }
 
-// Calculate the intersection of a line segment and a plane and returns false
-// if they are parallel or the intersection is outside the line segment.
-inline bool lcLinePlaneIntersection(lcVector3* Intersection, const lcVector3& Start, const lcVector3& End, const lcVector4& Plane)
+inline bool lcLineSegmentPlaneIntersection(lcVector3* Intersection, const lcVector3& Start, const lcVector3& End, const lcVector4& Plane)
 {
 	lcVector3 Dir = End - Start;
 	lcVector3 PlaneNormal(Plane[0], Plane[1], Plane[2]);
@@ -1711,7 +1723,7 @@ inline void lcPolygonPlaneClip(lcVector3* InPoints, int NumInPoints, lcVector3*
 			else
 			{
 				// Outside, inside.
-				lcLinePlaneIntersection(&i, *s, *p, Plane);
+				lcLineSegmentPlaneIntersection(&i, *s, *p, Plane);
 
 				OutPoints[*NumOutPoints] = i;
 				*NumOutPoints = *NumOutPoints + 1;
@@ -1724,7 +1736,7 @@ inline void lcPolygonPlaneClip(lcVector3* InPoints, int NumInPoints, lcVector3*
 			if (lcDot3(*s, Plane) + Plane[3] <= 0)
 			{
 				// Inside, outside.
-				lcLinePlaneIntersection(&i, *s, *p, Plane);
+				lcLineSegmentPlaneIntersection(&i, *s, *p, Plane);
 
 				OutPoints[*NumOutPoints] = i;
 				*NumOutPoints = *NumOutPoints + 1;
