@@ -1,38 +1,17 @@
-#ifndef _LC_PARTSELECTIONWIDGET_H_
-#define _LC_PARTSELECTIONWIDGET_H_
+#pragma once
+
+#include "lc_context.h"
 
 class lcPartSelectionListModel;
 class lcPartSelectionListView;
 
-class lcPartSelectionFilterModel : public QSortFilterProxyModel
-{
-	Q_OBJECT
-
-public:
-	lcPartSelectionFilterModel(QObject* Parent);
-
-	void SetFilter(const QString& Filter);
-	void SetShowDecoratedParts(bool Show);
-
-	bool GetShowDecoratedParts() const
-	{
-		return mShowDecoratedParts;
-	}
-
-protected:
-	virtual bool filterAcceptsRow(int SourceRow, const QModelIndex& SourceParent) const;
-
-	QByteArray mFilter;
-	bool mShowDecoratedParts;
-};
-
 class lcPartSelectionItemDelegate : public QStyledItemDelegate
 {
 	Q_OBJECT
 
 public:
-	lcPartSelectionItemDelegate(QObject* Parent, lcPartSelectionListModel* ListModel, lcPartSelectionFilterModel* FilterModel)
-		: QStyledItemDelegate(Parent), mListModel(ListModel), mFilterModel(FilterModel)
+	lcPartSelectionItemDelegate(QObject* Parent, lcPartSelectionListModel* ListModel)
+		: QStyledItemDelegate(Parent), mListModel(ListModel)
 	{
 	}
 
@@ -41,7 +20,6 @@ public:
 
 protected:
 	lcPartSelectionListModel* mListModel;
-	lcPartSelectionFilterModel* mFilterModel;
 };
 
 class lcPartSelectionListModel : public QAbstractListModel
@@ -67,6 +45,11 @@ public:
 		return mParts[Row].first;
 	}
 
+	bool GetShowDecoratedParts() const
+	{
+		return mShowDecoratedParts;
+	}
+
 	int GetIconSize() const
 	{
 		return mIconSize;
@@ -94,7 +77,9 @@ public:
 	void SetCategory(int CategoryIndex);
 	void SetModelsCategory();
 	void SetCurrentModelCategory();
+	void SetFilter(const QString& Filter);
 	void RequestPreview(int InfoIndex);
+	void SetShowDecoratedParts(bool Show);
 	void SetIconSize(int Size);
 	void SetShowPartNames(bool Show);
 
@@ -113,6 +98,9 @@ protected:
 	int mColorIndex;
 	bool mShowPartNames;
 	bool mListMode;
+	bool mShowDecoratedParts;
+	QByteArray mFilter;
+	std::pair<lcFramebuffer, lcFramebuffer> mRenderFramebuffer;
 };
 
 class lcPartSelectionListView : public QListView
@@ -126,7 +114,7 @@ public:
 
 	PieceInfo* GetCurrentPart() const
 	{
-		return mListModel->GetPieceInfo(mFilterModel->mapToSource(currentIndex()));
+		return mListModel->GetPieceInfo(currentIndex());
 	}
 
 	lcPartSelectionListModel* GetListModel() const
@@ -134,11 +122,6 @@ public:
 		return mListModel;
 	}
 
-	lcPartSelectionFilterModel* GetFilterModel() const
-	{
-		return mFilterModel;
-	}
-
 	void UpdateViewMode();
 
 public slots:
@@ -157,7 +140,6 @@ protected:
 	void SetIconSize(int Size);
 
 	lcPartSelectionListModel* mListModel;
-	lcPartSelectionFilterModel* mFilterModel;
 };
 
 class lcPartSelectionWidget : public QWidget
@@ -200,5 +182,3 @@ protected:
 	lcPartSelectionListView* mPartsWidget;
 	QSplitter* mSplitter;
 };
-
-#endif // _LC_PARTSELECTIONWIDGET_H_
