@@ -20,32 +20,34 @@ void lcSynthInit()
 
 	lcHoseInfo HoseInfo[] =
 	{
-		{ "72504",  LC_SYNTH_PIECE_RIBBED_HOSE,     31.25f,   4 }, // Technic Ribbed Hose  2L
-		{ "72706",  LC_SYNTH_PIECE_RIBBED_HOSE,     50.00f,   7 }, // Technic Ribbed Hose  3L
-		{ "71952",  LC_SYNTH_PIECE_RIBBED_HOSE,     75.00f,  11 }, // Technic Ribbed Hose  4L
-		{ "71944",  LC_SYNTH_PIECE_RIBBED_HOSE,    112.50f,  17 }, // Technic Ribbed Hose  6L
-		{ "71951",  LC_SYNTH_PIECE_RIBBED_HOSE,    143.75f,  22 }, // Technic Ribbed Hose  8L
-		{ "71986",  LC_SYNTH_PIECE_RIBBED_HOSE,    212.50f,  33 }, // Technic Ribbed Hose 11L
-		{ "43675",  LC_SYNTH_PIECE_RIBBED_HOSE,    375.00f,  58 }, // Technic Ribbed Hose 19L
-		{ "32580",  LC_SYNTH_PIECE_FLEXIBLE_AXLE,  120.00f,  15 }, // Technic Axle Flexible  7
-		{ "32199",  LC_SYNTH_PIECE_FLEXIBLE_AXLE,  200.00f,  35 }, // Technic Axle Flexible 11
-		{ "55709",  LC_SYNTH_PIECE_FLEXIBLE_AXLE,  200.00f,  35 }, // Technic Axle Flexible 11
-		{ "32200",  LC_SYNTH_PIECE_FLEXIBLE_AXLE,  220.00f,  40 }, // Technic Axle Flexible 12
-		{ "32201",  LC_SYNTH_PIECE_FLEXIBLE_AXLE,  260.00f,  50 }, // Technic Axle Flexible 14
-		{ "32202",  LC_SYNTH_PIECE_FLEXIBLE_AXLE,  300.00f,  60 }, // Technic Axle Flexible 16
-		{ "32235",  LC_SYNTH_PIECE_FLEXIBLE_AXLE,  360.00f,  75 }, // Technic Axle Flexible 19
-		{ "76384",  LC_SYNTH_PIECE_STRING_BRAIDED, 200.00f,  46 }, // String Braided 11L with End Studs
-		{ "75924",  LC_SYNTH_PIECE_STRING_BRAIDED, 400.00f,  96 }, // String Braided 21L with End Studs
-		{ "572C02", LC_SYNTH_PIECE_STRING_BRAIDED, 800.00f, 196 }, // String Braided 41L with End Studs
-		{ "73129",  LC_SYNTH_PIECE_SHOCK_ABSORBER, 110.00f,   1 }, // Technic Shock Absorber 6.5L 
-		{ "41838",  LC_SYNTH_PIECE_SHOCK_ABSORBER, 110.00f,   1 }, // Technic Shock Absorber 6.5L Soft
-		{ "76138",  LC_SYNTH_PIECE_SHOCK_ABSORBER, 110.00f,   1 }, // Technic Shock Absorber 6.5L Stiff
-		{ "76537",  LC_SYNTH_PIECE_SHOCK_ABSORBER, 110.00f,   1 }  // Technic Shock Absorber 6.5L Extra Stiff
+		{ "72504.dat",    lcSynthType::RIBBED_HOSE,     31.25f,   4 }, // Technic Ribbed Hose  2L
+		{ "72706.dat",    lcSynthType::RIBBED_HOSE,     50.00f,   7 }, // Technic Ribbed Hose  3L
+		{ "71952.dat",    lcSynthType::RIBBED_HOSE,     75.00f,  11 }, // Technic Ribbed Hose  4L
+		{ "71944.dat",    lcSynthType::RIBBED_HOSE,    112.50f,  17 }, // Technic Ribbed Hose  6L
+		{ "71951.dat",    lcSynthType::RIBBED_HOSE,    143.75f,  22 }, // Technic Ribbed Hose  8L
+		{ "71986.dat",    lcSynthType::RIBBED_HOSE,    212.50f,  33 }, // Technic Ribbed Hose 11L
+		{ "43675.dat",    lcSynthType::RIBBED_HOSE,    375.00f,  58 }, // Technic Ribbed Hose 19L
+		{ "32580.dat",    lcSynthType::FLEXIBLE_AXLE,  120.00f,  15 }, // Technic Axle Flexible  7
+		{ "32199.dat",    lcSynthType::FLEXIBLE_AXLE,  200.00f,  35 }, // Technic Axle Flexible 11
+		{ "55709.dat",    lcSynthType::FLEXIBLE_AXLE,  200.00f,  35 }, // Technic Axle Flexible 11
+		{ "32200.dat",    lcSynthType::FLEXIBLE_AXLE,  220.00f,  40 }, // Technic Axle Flexible 12
+		{ "32201.dat",    lcSynthType::FLEXIBLE_AXLE,  260.00f,  50 }, // Technic Axle Flexible 14
+		{ "32202.dat",    lcSynthType::FLEXIBLE_AXLE,  300.00f,  60 }, // Technic Axle Flexible 16
+		{ "32235.dat",    lcSynthType::FLEXIBLE_AXLE,  360.00f,  75 }, // Technic Axle Flexible 19
+		{ "76384.dat",    lcSynthType::STRING_BRAIDED, 200.00f,  46 }, // String Braided 11L with End Studs
+		{ "75924.dat",    lcSynthType::STRING_BRAIDED, 400.00f,  96 }, // String Braided 21L with End Studs
+		{ "572C02.dat",   lcSynthType::STRING_BRAIDED, 800.00f, 196 }, // String Braided 41L with End Studs
+		{ "73129.dat",    lcSynthType::SHOCK_ABSORBER, 110.00f,   1 }, // Technic Shock Absorber 6.5L
+		{ "41838.dat",    lcSynthType::SHOCK_ABSORBER, 110.00f,   1 }, // Technic Shock Absorber 6.5L Soft
+		{ "76138.dat",    lcSynthType::SHOCK_ABSORBER, 110.00f,   1 }, // Technic Shock Absorber 6.5L Stiff
+		{ "76537.dat",    lcSynthType::SHOCK_ABSORBER, 110.00f,   1 }, // Technic Shock Absorber 6.5L Extra Stiff
+		{ "61927C01.dat", lcSynthType::ACTUATOR,       270.00f,   1 }, // Technic Power Functions Linear Actuator (Extended)
+		{ "61927.dat",    lcSynthType::ACTUATOR,       170.00f,   1 }  // Technic Power Functions Linear Actuator (Contracted)
 	};
 
 	for (unsigned int InfoIdx = 0; InfoIdx < sizeof(HoseInfo) / sizeof(HoseInfo[0]); InfoIdx++)
 	{
-		PieceInfo* Info = Library->FindPiece(HoseInfo[InfoIdx].PartID, NULL, false, false);
+		PieceInfo* Info = Library->FindPiece(HoseInfo[InfoIdx].PartID, nullptr, false, false);
 
 		if (Info)
 			Info->SetSynthInfo(new lcSynthInfo(HoseInfo[InfoIdx].Type, HoseInfo[InfoIdx].Length, HoseInfo[InfoIdx].NumSections, Info));
@@ -64,28 +66,29 @@ lcSynthInfo::lcSynthInfo(lcSynthType Type, float Length, int NumSections, PieceI
 
 	switch (mType)
 	{
-	case LC_SYNTH_PIECE_RIBBED_HOSE:
+	case lcSynthType::RIBBED_HOSE:
 		EdgeSectionLength = 6.25f;
 		MidSectionLength = 6.25f;
 		mRigidEdges = false;
 		mCurve = true;
 		break;
 
-	case LC_SYNTH_PIECE_FLEXIBLE_AXLE:
+	case lcSynthType::FLEXIBLE_AXLE:
 		EdgeSectionLength = 30.0f;
 		MidSectionLength = 4.0f;
 		mRigidEdges = true;
 		mCurve = true;
 		break;
 
-	case LC_SYNTH_PIECE_STRING_BRAIDED:
+	case lcSynthType::STRING_BRAIDED:
 		EdgeSectionLength = 8.0f;
 		MidSectionLength = 4.0f;
 		mRigidEdges = true;
 		mCurve = true;
 		break;
 
-	case LC_SYNTH_PIECE_SHOCK_ABSORBER:
+	case lcSynthType::SHOCK_ABSORBER:
+	case lcSynthType::ACTUATOR:
 		EdgeSectionLength = 0.0f;
 		MidSectionLength = 0.0f;
 		mRigidEdges = false;
@@ -93,7 +96,7 @@ lcSynthInfo::lcSynthInfo(lcSynthType Type, float Length, int NumSections, PieceI
 		break;
 	}
 
-	if (mType != LC_SYNTH_PIECE_SHOCK_ABSORBER)
+	if (mType != lcSynthType::SHOCK_ABSORBER && mType != lcSynthType::ACTUATOR)
 	{
 		mStart.Transform = lcMatrix44(lcMatrix33(lcVector3(0.0f, 0.0f, 1.0f), lcVector3(1.0f, 0.0f, 0.0f), lcVector3(0.0f, 1.0f, 0.0f)), lcVector3(0.0f, 0.0f, 0.0f));
 		mMiddle.Transform = lcMatrix44Identity();
@@ -119,19 +122,20 @@ void lcSynthInfo::GetDefaultControlPoints(lcArray<lcPieceControlPoint>& ControlP
 
 	switch (mType)
 	{
-	case LC_SYNTH_PIECE_RIBBED_HOSE:
+	case lcSynthType::RIBBED_HOSE:
 		Scale = 80.0f;
 		break;
 
-	case LC_SYNTH_PIECE_FLEXIBLE_AXLE:
+	case lcSynthType::FLEXIBLE_AXLE:
 		Scale = 12.0f;
 		break;
 
-	case LC_SYNTH_PIECE_STRING_BRAIDED:
+	case lcSynthType::STRING_BRAIDED:
 		Scale = 12.0f;
 		break;
 
-	case LC_SYNTH_PIECE_SHOCK_ABSORBER:
+	case lcSynthType::SHOCK_ABSORBER:
+	case lcSynthType::ACTUATOR:
 		Scale = 1.0f;
 		break;
 	}
@@ -139,15 +143,22 @@ void lcSynthInfo::GetDefaultControlPoints(lcArray<lcPieceControlPoint>& ControlP
 	float HalfLength = mLength / 2.0f;
 	Scale = lcMin(Scale, HalfLength);
 
-	if (mType != LC_SYNTH_PIECE_SHOCK_ABSORBER)
+	switch (mType)
 	{
+	default:
 		ControlPoints[0].Transform = lcMatrix44Translation(lcVector3(-HalfLength, 0.0f, 0.0f));
 		ControlPoints[1].Transform = lcMatrix44Translation(lcVector3( HalfLength, 0.0f, 0.0f));
-	}
-	else
-	{
+		break;
+
+	case lcSynthType::SHOCK_ABSORBER:
 		ControlPoints[0].Transform = lcMatrix44Translation(lcVector3(0.0f, 0.0f, -mLength));
 		ControlPoints[1].Transform = lcMatrix44Translation(lcVector3(0.0f, 0.0f, 0.0f));
+		break;
+
+	case lcSynthType::ACTUATOR:
+		ControlPoints[0].Transform = lcMatrix44(lcMatrix33(lcVector3(1.0f, 0.0f, 0.0f), lcVector3(0.0f, 0.0f, -1.0f), lcVector3(0.0f, 1.0f, 0.0f)), lcVector3(0.0f, 0.0f, 0.0f));
+		ControlPoints[1].Transform = lcMatrix44(lcMatrix33(lcVector3(1.0f, 0.0f, 0.0f), lcVector3(0.0f, 0.0f, -1.0f), lcVector3(0.0f, 1.0f, 0.0f)), lcVector3(0.0f, mLength, 0.0f));
+		break;
 	}
 
 	ControlPoints[0].Scale = Scale;
@@ -334,7 +345,7 @@ void lcSynthInfo::AddRibbedHoseParts(lcMemFile& File, const lcArray<lcMatrix44>&
 		lcMatrix33 Transform(lcMul(lcMatrix33Scale(lcVector3(1.0f, -1.0f, 1.0f)), lcMatrix33(Sections[SectionIdx])));
 		lcVector3 Offset = Sections[SectionIdx].GetTranslation();
 
-		sprintf(Line, "1 16 %f %f %f %f %f %f %f %f %f %f %f %f 79.DAT\n", Offset[0], Offset[1], Offset[2], Transform[0][0], Transform[1][0], Transform[2][0],
+		sprintf(Line, "1 16 %f %f %f %f %f %f %f %f %f %f %f %f 79.dat\n", Offset[0], Offset[1], Offset[2], Transform[0][0], Transform[1][0], Transform[2][0],
 				Transform[0][1], Transform[1][1], Transform[2][1], Transform[0][2], Transform[1][2], Transform[2][2]);
 
 		File.WriteBuffer(Line, strlen(Line));
@@ -344,7 +355,7 @@ void lcSynthInfo::AddRibbedHoseParts(lcMemFile& File, const lcArray<lcMatrix44>&
 	{
 		const lcMatrix44& Transform = Sections[SectionIdx];
 
-		sprintf(Line, "1 16 %f %f %f %f %f %f %f %f %f %f %f %f 80.DAT\n", Transform[3][0], Transform[3][1], Transform[3][2], Transform[0][0], Transform[1][0], Transform[2][0],
+		sprintf(Line, "1 16 %f %f %f %f %f %f %f %f %f %f %f %f 80.dat\n", Transform[3][0], Transform[3][1], Transform[3][2], Transform[0][0], Transform[1][0], Transform[2][0],
 				Transform[0][1], Transform[1][1], Transform[2][1], Transform[0][2], Transform[1][2], Transform[2][2]);
 
 		File.WriteBuffer(Line, strlen(Line));
@@ -355,7 +366,7 @@ void lcSynthInfo::AddRibbedHoseParts(lcMemFile& File, const lcArray<lcMatrix44>&
 		lcMatrix33 Transform(Sections[SectionIdx]);
 		lcVector3 Offset = lcMul31(lcVector3(0.0f, -6.25f, 0.0f), Sections[SectionIdx]);
 
-		sprintf(Line, "1 16 %f %f %f %f %f %f %f %f %f %f %f %f 79.DAT\n", Offset[0], Offset[1], Offset[2], Transform[0][0], Transform[1][0], Transform[2][0],
+		sprintf(Line, "1 16 %f %f %f %f %f %f %f %f %f %f %f %f 79.dat\n", Offset[0], Offset[1], Offset[2], Transform[0][0], Transform[1][0], Transform[2][0],
 				Transform[0][1], Transform[1][1], Transform[2][1], Transform[0][2], Transform[1][2], Transform[2][2]);
 
 		File.WriteBuffer(Line, strlen(Line));
@@ -379,7 +390,7 @@ void lcSynthInfo::AddFlexibleAxleParts(lcMemFile& File, lcLibraryMeshData& MeshD
 
 	const char* EdgeParts[6] =
 	{
-		"STUD3A.DAT", "S/FAXLE1.DAT", "S/FAXLE2.DAT", "S/FAXLE3.DAT", "S/FAXLE4.DAT", "S/FAXLE5.DAT"
+		"stud3a.dat", "s/faxle1.dat", "s/faxle2.dat", "s/faxle3.dat", "s/faxle4.dat", "s/faxle5.dat"
 	};
 
 	for (int PartIdx = 0; PartIdx < NumEdgeParts; PartIdx++)
@@ -415,8 +426,8 @@ void lcSynthInfo::AddFlexibleAxleParts(lcMemFile& File, lcLibraryMeshData& MeshD
 	};
 
 	int BaseVertex;
-	lcVertex* VertexBuffer;
-	lcuint32* IndexBuffer;
+	lcLibraryMeshVertex* VertexBuffer;
+	quint32* IndexBuffer;
 	MeshData.AddVertices(LC_MESHDATA_SHARED, 16 * (Sections.GetSize() - 1), &BaseVertex, &VertexBuffer);
 	MeshData.AddIndices(LC_MESHDATA_SHARED, LC_MESH_LINES, 24, 2 * 12 * (Sections.GetSize() - 2), &IndexBuffer);
 
@@ -480,7 +491,7 @@ void lcSynthInfo::AddStringBraidedParts(lcMemFile& File, lcLibraryMeshData& Mesh
 		lcMatrix33 Transform(Sections[SectionIdx]);
 		lcVector3 Offset = lcMul31(lcVector3(-8.0f, 0.0f, 0.0f), Sections[SectionIdx]);
 
-		sprintf(Line, "1 16 %f %f %f %f %f %f %f %f %f %f %f %f 572A.DAT\n", Offset[0], Offset[1], Offset[2], Transform[0][0], Transform[1][0], Transform[2][0],
+		sprintf(Line, "1 16 %f %f %f %f %f %f %f %f %f %f %f %f 572A.dat\n", Offset[0], Offset[1], Offset[2], Transform[0][0], Transform[1][0], Transform[2][0],
 				Transform[0][1], Transform[1][1], Transform[2][1], Transform[0][2], Transform[1][2], Transform[2][2]);
 
 		File.WriteBuffer(Line, strlen(Line));
@@ -494,8 +505,8 @@ void lcSynthInfo::AddStringBraidedParts(lcMemFile& File, lcLibraryMeshData& Mesh
 	};
 
 	int BaseVertex;
-	lcVertex* VertexBuffer;
-	lcuint32* IndexBuffer;
+	lcLibraryMeshVertex* VertexBuffer;
+	quint32* IndexBuffer;
 	MeshData.AddVertices(LC_MESHDATA_SHARED, NumBraids * ((Sections.GetSize() - 2) * NumSegments + 1), &BaseVertex, &VertexBuffer);
 	MeshData.AddIndices(LC_MESHDATA_SHARED, LC_MESH_LINES, 24, NumBraids * (Sections.GetSize() - 2) * NumSegments * 2, &IndexBuffer);
 
@@ -583,7 +594,7 @@ void lcSynthInfo::AddStringBraidedParts(lcMemFile& File, lcLibraryMeshData& Mesh
 		lcMatrix33 Transform(Sections[SectionIdx]);
 		lcVector3 Offset = lcMul31(lcVector3(8.0f, 0.0f, 0.0f), Sections[SectionIdx]);
 
-		sprintf(Line, "1 16 %f %f %f %f %f %f %f %f %f %f %f %f 572A.DAT\n", Offset[0], Offset[1], Offset[2], Transform[0][0], Transform[1][0], Transform[2][0],
+		sprintf(Line, "1 16 %f %f %f %f %f %f %f %f %f %f %f %f 572A.dat\n", Offset[0], Offset[1], Offset[2], Transform[0][0], Transform[1][0], Transform[2][0],
 				Transform[0][1], Transform[1][1], Transform[2][1], Transform[0][2], Transform[1][2], Transform[2][2]);
 
 		File.WriteBuffer(Line, strlen(Line));
@@ -596,30 +607,46 @@ void lcSynthInfo::AddShockAbsorberParts(lcMemFile& File, lcArray<lcMatrix44>& Se
 	lcVector3 Offset;
 
 	Offset = Sections[0].GetTranslation();
-	sprintf(Line, "1 0 %f %f %f 1 0 0 0 1 0 0 0 1 4254.DAT\n", Offset[0], Offset[1], Offset[2]);
+	sprintf(Line, "1 0 %f %f %f 1 0 0 0 1 0 0 0 1 4254.dat\n", Offset[0], Offset[1], Offset[2]);
 	File.WriteBuffer(Line, strlen(Line));
 
 	Offset = Sections[1].GetTranslation();
-	sprintf(Line, "1 16 %f %f %f 1 0 0 0 1 0 0 0 1 4255.DAT\n", Offset[0], Offset[1], Offset[2]);
+	sprintf(Line, "1 16 %f %f %f 1 0 0 0 1 0 0 0 1 4255.dat\n", Offset[0], Offset[1], Offset[2]);
 	File.WriteBuffer(Line, strlen(Line));
 
 	float Distance = Sections[0].GetTranslation().y - Sections[1].GetTranslation().y;
 	float Scale = (Distance - 66.0f) / 44.0f;
 	const char* SpringPart;
 
-	if (!strcmp(mPieceInfo->m_strName, "73129"))
+	if (!qstricmp(mPieceInfo->mFileName, "73129.dat"))
 		SpringPart = "70038";
-	else if (!strcmp(mPieceInfo->m_strName, "41838"))
+	else if (!qstricmp(mPieceInfo->mFileName, "41838"))
 		SpringPart = "41837";
-	else if (!strcmp(mPieceInfo->m_strName, "76138"))
+	else if (!qstricmp(mPieceInfo->mFileName, "76138"))
 		SpringPart = "71953";
-	else if (!strcmp(mPieceInfo->m_strName, "76537"))
+	else if (!qstricmp(mPieceInfo->mFileName, "76537"))
 		SpringPart = "22977";
 	else
 		return;
 
 	Offset = Sections[0].GetTranslation();
-	sprintf(Line, "1 494 %f %f %f 1 0 0 0 %f 0 0 0 1 %s.DAT\n", Offset[0], Offset[1] - 10 - 44.0f * Scale, Offset[2], Scale, SpringPart);
+	sprintf(Line, "1 494 %f %f %f 1 0 0 0 %f 0 0 0 1 %s.dat\n", Offset[0], Offset[1] - 10 - 44.0f * Scale, Offset[2], Scale, SpringPart);
+	File.WriteBuffer(Line, strlen(Line));
+}
+
+void lcSynthInfo::AddActuatorParts(lcMemFile& File, lcArray<lcMatrix44>& Sections) const
+{
+	char Line[256];
+	lcVector3 Offset;
+
+	Offset = Sections[0].GetTranslation();
+	sprintf(Line, "1 25 %f %f %f 0 1 0 -1 0 0 0 0 1 47157.dat\n", Offset[0], Offset[1], Offset[2]);
+	File.WriteBuffer(Line, strlen(Line));
+	sprintf(Line, "1 16 %f %f %f 1 0 0 0 1 0 0 0 1 62271c01.dat\n", Offset[0], Offset[1], Offset[2]);
+	File.WriteBuffer(Line, strlen(Line));
+
+	Offset = Sections[1].GetTranslation();
+	sprintf(Line, "1 72 %f %f %f 1 0 0 0 1 0 0 0 1 62274c01.dat\n", Offset[0], Offset[1], Offset[2]);
 	File.WriteBuffer(Line, strlen(Line));
 }
 
@@ -628,30 +655,34 @@ lcMesh* lcSynthInfo::CreateMesh(const lcArray<lcPieceControlPoint>& ControlPoint
 	lcArray<lcMatrix44> Sections;
 
 	if (mCurve)
-		CalculateCurveSections(ControlPoints, Sections, NULL, NULL);
+		CalculateCurveSections(ControlPoints, Sections, nullptr, nullptr);
 	else
-		CalculateLineSections(ControlPoints, Sections, NULL, NULL);
+		CalculateLineSections(ControlPoints, Sections, nullptr, nullptr);
 
 	lcLibraryMeshData MeshData;
 	lcMemFile File; // todo: rewrite this to pass the parts directly
 
 	switch (mType)
 	{
-	case LC_SYNTH_PIECE_RIBBED_HOSE:
+	case lcSynthType::RIBBED_HOSE:
 		AddRibbedHoseParts(File, Sections);
 		break;
 
-	case LC_SYNTH_PIECE_FLEXIBLE_AXLE:
+	case lcSynthType::FLEXIBLE_AXLE:
 		AddFlexibleAxleParts(File, MeshData, Sections);
 		break;
 
-	case LC_SYNTH_PIECE_STRING_BRAIDED:
+	case lcSynthType::STRING_BRAIDED:
 		AddStringBraidedParts(File, MeshData, Sections);
 		break;
 
-	case LC_SYNTH_PIECE_SHOCK_ABSORBER:
+	case lcSynthType::SHOCK_ABSORBER:
 		AddShockAbsorberParts(File, Sections);
 		break;
+
+	case lcSynthType::ACTUATOR:
+		AddActuatorParts(File, Sections);
+		break;
 	}
 
 	File.WriteU8(0);
@@ -659,10 +690,10 @@ lcMesh* lcSynthInfo::CreateMesh(const lcArray<lcPieceControlPoint>& ControlPoint
 	lcArray<lcLibraryTextureMap> TextureStack;
 	File.Seek(0, SEEK_SET);
 
-	if (lcGetPiecesLibrary()->ReadMeshData(File, lcMatrix44Identity(), 16, TextureStack, MeshData, LC_MESHDATA_SHARED, false))
-		return lcGetPiecesLibrary()->CreateMesh(NULL, MeshData);
+	if (lcGetPiecesLibrary()->ReadMeshData(File, lcMatrix44Identity(), 16, false, TextureStack, MeshData, LC_MESHDATA_SHARED, false, nullptr, false))
+		return lcGetPiecesLibrary()->CreateMesh(nullptr, MeshData);
 
-	return NULL;
+	return nullptr;
 }
 
 struct lcSynthInsertParam
