@@ -112,7 +112,7 @@ bool TexFont::Load(lcContext* Context)
 	mFontHeight = 16;
 
 	glGenTextures(1, &mTexture);
-	Context->BindTexture(mTexture);
+	Context->BindTexture2D(mTexture);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
