@@ -19,9 +19,10 @@ View::View(lcModel* Model)
 	mHighlight = false;
 	memset(mGridSettings, 0, sizeof(mGridSettings));
 
-	mDragState = LC_DRAGSTATE_NONE;
+	mDragState = lcDragState::NONE;
 	mTrackButton = LC_TRACKBUTTON_NONE;
 	mTrackTool = LC_TRACKTOOL_NONE;
+	mTrackToolFromOverlay = false;
 
 	View* ActiveView = gMainWindow->GetActiveView();
 	if (ActiveView)
@@ -275,6 +276,8 @@ void View::SetViewpoint(lcViewpoint Viewpoint)
 	mCamera->SetViewpoint(Viewpoint);
 	ZoomExtents();
 	Redraw();
+
+	gMainWindow->UpdateCurrentCamera(-1);
 }
 
 void View::SetCameraAngles(float Latitude, float Longitude)
@@ -293,6 +296,8 @@ void View::SetDefaultCamera()
 		mCamera = new lcCamera(true);
 
 	mCamera->SetViewpoint(LC_VIEWPOINT_HOME);
+
+	gMainWindow->UpdateCurrentCamera(-1);
 }
 
 lcMatrix44 View::GetProjectionMatrix() const
@@ -518,7 +523,7 @@ lcMatrix44 View::GetPieceInsertPosition(bool IgnoreSelected, PieceInfo* Info) co
 	lcVector3 Intersection;
 
 	const lcBoundingBox& BoundingBox = Info->GetBoundingBox();
-	if (lcLinePlaneIntersection(&Intersection, ClickPoints[0], ClickPoints[1], lcVector4(0, 0, 1, BoundingBox.Min.z)))
+	if (lcLineSegmentPlaneIntersection(&Intersection, ClickPoints[0], ClickPoints[1], lcVector4(0, 0, 1, BoundingBox.Min.z)))
 	{
 		Intersection = mModel->SnapPosition(Intersection);
 		return lcMatrix44Translation(Intersection);
@@ -1611,7 +1616,7 @@ void View::DrawAxes()
 		19, 20, 21, 19, 21, 22, 19, 22, 23, 19, 23, 24, 19, 24, 25, 19, 25, 26, 19, 26, 27, 19, 27, 20
 	};
 
-	lcMatrix44 TranslationMatrix = lcMatrix44Translation(lcVector3(25.375f, 25.375f, 0.0f));
+	lcMatrix44 TranslationMatrix = lcMatrix44Translation(lcVector3(30.375f, 30.375f, 0.0f));
 	lcMatrix44 WorldViewMatrix = mCamera->mWorldView;
 	WorldViewMatrix.SetTranslation(lcVector3(0, 0, 0));
 
@@ -1745,6 +1750,9 @@ lcTool View::GetCurrentTool() const
 
 lcTrackTool View::GetOverrideTrackTool(Qt::MouseButton Button) const
 {
+	if (mTrackToolFromOverlay)
+		return LC_TRACKTOOL_NONE;
+
 	lcTool OverrideTool = gMouseShortcuts.GetTool(Button, mInputState.Modifiers);
 
 	if (OverrideTool == LC_NUM_TOOLS)
@@ -1785,18 +1793,32 @@ float View::GetOverlayScale() const
 	return Dist.Length() * 5.0f;
 }
 
-void View::BeginPieceDrag()
+void View::BeginDrag(lcDragState DragState)
 {
-	mDragState = LC_DRAGSTATE_PIECE;
+	mDragState = DragState;
 	UpdateTrackTool();
 }
 
-void View::EndPieceDrag(bool Accept)
+void View::EndDrag(bool Accept)
 {
 	if (Accept)
-		mModel->InsertPieceToolClicked(GetPieceInsertPosition(false, gMainWindow->GetCurrentPieceInfo()));
+	{
+		switch (mDragState)
+		{
+		case lcDragState::NONE:
+			break;
 
-	mDragState = LC_DRAGSTATE_NONE;
+		case lcDragState::PIECE:
+			mModel->InsertPieceToolClicked(GetPieceInsertPosition(false, gMainWindow->GetCurrentPieceInfo()));
+			break;
+
+		case lcDragState::COLOR:
+			mModel->PaintToolClicked(FindObjectUnderPointer(true, false).Object);
+			break;
+		}
+	}
+
+	mDragState = lcDragState::NONE;
 	UpdateTrackTool();
 }
 
@@ -1835,6 +1857,7 @@ void View::UpdateTrackTool()
 	int x = mInputState.x;
 	int y = mInputState.y;
 	bool Redraw = false;
+	mTrackToolFromOverlay = false;
 
 	switch (CurrentTool)
 	{
@@ -1873,7 +1896,6 @@ void View::UpdateTrackTool()
 			if (!mModel->GetMoveRotateTransform(OverlayCenter, RelativeRotation))
 				break;
 
-			// Intersect the mouse with the 3 planes.
 			lcVector3 PlaneNormals[3] =
 			{
 				lcVector3(1.0f, 0.0f, 0.0f),
@@ -1908,7 +1930,7 @@ void View::UpdateTrackTool()
 				lcVector4 Plane(PlaneNormals[AxisIndex], -lcDot(PlaneNormals[AxisIndex], OverlayCenter));
 				lcVector3 Intersection;
 
-				if (!lcLinePlaneIntersection(&Intersection, Start, End, Plane))
+				if (!lcLineSegmentPlaneIntersection(&Intersection, Start, End, Plane))
 					continue;
 
 				float IntersectionDistance = lcLengthSquared(Intersection - Start);
@@ -2028,6 +2050,7 @@ void View::UpdateTrackTool()
 				}
 			}
 
+			mTrackToolFromOverlay = NewTrackTool != LC_TRACKTOOL_MOVE_XYZ && NewTrackTool != LC_TRACKTOOL_SELECT;
 			Redraw = true;
 		}
 		break;
@@ -2166,6 +2189,7 @@ void View::UpdateTrackTool()
 								break;
 							}
 
+							mTrackToolFromOverlay = true;
 							Dist *= r;
 							break;
 						}
@@ -2213,10 +2237,16 @@ void View::UpdateTrackTool()
 			if ((d < r + SquareSize) && (d > r - SquareSize))
 			{
 				if ((cx - x < SquareSize) && (cx - x > -SquareSize))
+				{
 					NewTrackTool = LC_TRACKTOOL_ORBIT_Y;
+					mTrackToolFromOverlay = true;
+				}
 
 				if ((cy - y < SquareSize) && (cy - y > -SquareSize))
+				{
 					NewTrackTool = LC_TRACKTOOL_ORBIT_X;
+					mTrackToolFromOverlay = true;
+				}
 			}
 			else
 			{
@@ -2242,13 +2272,17 @@ void View::UpdateTrackTool()
 
 	switch (mDragState)
 	{
-	case LC_DRAGSTATE_NONE:
+	case lcDragState::NONE:
 		break;
 
-	case LC_DRAGSTATE_PIECE:
+	case lcDragState::PIECE:
 		NewTrackTool = LC_TRACKTOOL_INSERT;
 		Redraw = true;
 		break;
+
+	case lcDragState::COLOR:
+		NewTrackTool = LC_TRACKTOOL_PAINT;
+		break;
 	}
 
 	if (NewTrackTool != mTrackTool)
@@ -2468,7 +2502,7 @@ void View::StopTracking(bool Accept)
 			lcVector4 Plane(PlaneNormal, -lcDot(PlaneNormal, Center));
 			lcVector3 Target, Corners[2];
 
-			if (lcLinePlaneIntersection(&Target, Points[0], Points[1], Plane) && lcLinePlaneIntersection(&Corners[0], Points[2], Points[3], Plane) && lcLinePlaneIntersection(&Corners[1], Points[3], Points[4], Plane))
+			if (lcLineSegmentPlaneIntersection(&Target, Points[0], Points[1], Plane) && lcLineSegmentPlaneIntersection(&Corners[0], Points[2], Points[3], Plane) && lcLineSegmentPlaneIntersection(&Corners[1], Points[3], Points[4], Plane))
 			{
 				float AspectRatio = (float)mWidth / (float)mHeight;
 				mModel->ZoomRegionToolClicked(mCamera, AspectRatio, Points[0], Target, Corners);
@@ -2804,11 +2838,11 @@ void View::OnMouseMove()
 				lcVector4 Plane(PlaneNormal, -lcDot(PlaneNormal, Center));
 				lcVector3 Intersection;
 
-				if (lcLinePlaneIntersection(&Intersection, CurrentStart, CurrentEnd, Plane))
+				if (lcLineSegmentPlaneIntersection(&Intersection, CurrentStart, CurrentEnd, Plane))
 				{
 					lcVector3 MoveStart;
 
-					if (lcLinePlaneIntersection(&MoveStart, MouseDownStart, MouseDownEnd, Plane))
+					if (lcLineSegmentPlaneIntersection(&MoveStart, MouseDownStart, MouseDownEnd, Plane))
 					{
 						lcVector3 Distance = Intersection - MoveStart;
 						Distance = lcMul(Distance, lcMatrix33AffineInverse(RelativeRotation));
@@ -2866,11 +2900,11 @@ void View::OnMouseMove()
 				lcVector4 Plane(PlaneNormal, -lcDot(PlaneNormal, Center));
 				lcVector3 Intersection;
 
-				if (lcLinePlaneIntersection(&Intersection, CurrentStart, CurrentEnd, Plane))
+				if (lcLineSegmentPlaneIntersection(&Intersection, CurrentStart, CurrentEnd, Plane))
 				{
 					lcVector3 MoveStart;
 
-					if (lcLinePlaneIntersection(&MoveStart, MouseDownStart, MouseDownEnd, Plane))
+					if (lcLineSegmentPlaneIntersection(&MoveStart, MouseDownStart, MouseDownEnd, Plane))
 					{
 						lcVector3 Distance = Intersection - MoveStart;
 						mModel->UpdateMoveTool(Distance, mTrackButton != LC_TRACKBUTTON_LEFT);
@@ -2983,15 +3017,18 @@ void View::OnMouseMove()
 
 			lcVector3 PlaneNormal(mCamera->mPosition - mCamera->mTargetPosition);
 			lcVector4 Plane(PlaneNormal, -lcDot(PlaneNormal, Center));
-			lcVector3 Intersection;
+			lcVector3 Intersection, MoveStart;
 
-			if (lcLinePlaneIntersection(&Intersection, CurrentStart, CurrentEnd, Plane))
+			if (!lcLineSegmentPlaneIntersection(&Intersection, CurrentStart, CurrentEnd, Plane) || !lcLineSegmentPlaneIntersection(&MoveStart, MouseDownStart, MouseDownEnd, Plane))
 			{
-				lcVector3 MoveStart;
+				Center = MouseDownStart + lcNormalize(MouseDownEnd - MouseDownStart) * 10.0f;
+				Plane = lcVector4(PlaneNormal, -lcDot(PlaneNormal, Center));
 
-				if (lcLinePlaneIntersection(&MoveStart, MouseDownStart, MouseDownEnd, Plane))
-					mModel->UpdatePanTool(mCamera, MoveStart - Intersection);
+				if (!lcLineSegmentPlaneIntersection(&Intersection, CurrentStart, CurrentEnd, Plane) || !lcLineSegmentPlaneIntersection(&MoveStart, MouseDownStart, MouseDownEnd, Plane))
+					break;
 			}
+
+			mModel->UpdatePanTool(mCamera, MoveStart - Intersection);
 		}
 		break;
 
