@@ -68,13 +68,16 @@ public:
 	virtual bool eventFilter(QObject* Object, QEvent* Event) override;
 
 public slots:
-	void DownloadFinished(lcHttpReply* Reply);
+#ifndef Q_OS_WIN
+	void DownloadFinished(QNetworkReply* Reply);
+#endif
 	void on_SearchButton_clicked();
 	void accept() override;
 	void Finished(int Result);
 
 protected:
 	lcHttpReply* RequestURL(const QString& URL);
+	void ProcessReply(lcHttpReply* Reply);
 
 #ifndef Q_OS_WIN
 	QNetworkAccessManager mNetworkManager;
